/*
* loglevel - https://github.com/pimterry/loglevel
*
* Copyright (c) 2013 Tim Perry
* Licensed under the MIT license.
*/
(function (root, definition) {
    "use strict";

    if (typeof module === 'object' && module.exports && typeof require === 'function') {
        module.exports = definition();
    } else if (typeof define === 'function' && typeof define.amd === 'object') {
        define(definition);
    } else {
        root.log = definition();
    }
}(this, function () {
    var noop = function() {};
    var undefinedType = "undefined";

    function realMethod(methodName) {
        if (typeof console === undefinedType) {
            return false; // We can't build a real method without a console to log to
        } else if (console[methodName] !== undefined) {
            return bindMethod(console, methodName);
        } else if (console.log !== undefined) {
            return bindMethod(console, 'log');
        } else {
            return noop;
        }
    }

    function bindMethod(obj, methodName) {
        var method = obj[methodName];
        if (typeof method.bind === 'function') {
            return method.bind(obj);
        } else {
            try {
                return Function.prototype.bind.call(method, obj);
            } catch (e) {
                // Missing bind shim or IE8 + Modernizr, fallback to wrapping
                return function() {
                    return Function.prototype.apply.apply(method, [obj, arguments]);
                };
            }
        }
    }

    // these private functions always need `this` to be set properly

    function enableLoggingWhenConsoleArrives(methodName, level, loggerName) {
        return function () {
            if (typeof console !== undefinedType) {
                replaceLoggingMethods.call(this, level, loggerName);
                this[methodName].apply(this, arguments);
            }
        };
    }

    function replaceLoggingMethods(level, loggerName) {
        for (var i = 0; i < logMethods.length; i++) {
            var methodName = logMethods[i];
            this[methodName] = (i < level) ?
                noop :
                this.methodFactory(methodName, level, loggerName);
        }
    }

    function defaultMethodFactory(methodName, level, loggerName) {
        return realMethod(methodName) ||
               enableLoggingWhenConsoleArrives.apply(this, arguments);
    }

    var logMethods = [
        "trace",
        "debug",
        "info",
        "warn",
        "error"
    ];

    function Logger(name, defaultLevel, factory) {
      var self = this;
      var currentLevel;
      var storageKey = "loglevel";
      if (name) {
        storageKey += ":" + name;
      }

      function persistLevelIfPossible(levelNum) {
          var levelName = (logMethods[levelNum] || 'silent').toUpperCase();

          // Use localStorage if available
          try {
              window.localStorage[storageKey] = levelName;
              return;
          } catch (ignore) {}

          // Use session cookie as fallback
          try {
              window.document.cookie =
                encodeURIComponent(storageKey) + "=" + levelName + ";";
          } catch (ignore) {}
      }

      function getPersistedLevel() {
          var storedLevel;

          try {
              storedLevel = window.localStorage[storageKey];
          } catch (ignore) {}

          if (typeof storedLevel === undefinedType) {
              try {
                  var cookie = window.document.cookie;
                  var location = cookie.indexOf(
                      encodeURIComponent(storageKey) + "=");
                  if (location) {
                      storedLevel = /^([^;]+)/.exec(cookie.slice(location))[1];
                  }
              } catch (ignore) {}
          }

          // If the stored level is not valid, treat it as if nothing was stored.
          if (self.levels[storedLevel] === undefined) {
              storedLevel = undefined;
          }

          return storedLevel;
      }

      /*
       *
       * Public API
       *
       */

      self.levels = { "TRACE": 0, "DEBUG": 1, "INFO": 2, "WARN": 3,
          "ERROR": 4, "SILENT": 5};

      self.methodFactory = factory || defaultMethodFactory;

      self.getLevel = function () {
          return currentLevel;
      };

      self.setLevel = function (level, persist) {
          if (typeof level === "string" && self.levels[level.toUpperCase()] !== undefined) {
              level = self.levels[level.toUpperCase()];
          }
          if (typeof level === "number" && level >= 0 && level <= self.levels.SILENT) {
              currentLevel = level;
              if (persist !== false) {  // defaults to true
                  persistLevelIfPossible(level);
              }
              replaceLoggingMethods.call(self, level, name);
              if (typeof console === undefinedType && level < self.levels.SILENT) {
                  return "No console available for logging";
              }
          } else {
              throw "log.setLevel() called with invalid level: " + level;
          }
      };

      self.setDefaultLevel = function (level) {
          if (!getPersistedLevel()) {
              self.setLevel(level, false);
          }
      };

      self.enableAll = function(persist) {
          self.setLevel(self.levels.TRACE, persist);
      };

      self.disableAll = function(persist) {
          self.setLevel(self.levels.SILENT, persist);
      };

      // Initialize with the right level
      var initialLevel = getPersistedLevel();
      if (initialLevel == null) {
          initialLevel = defaultLevel == null ? "WARN" : defaultLevel;
      }
      self.setLevel(initialLevel, false);
    }

    /*
     *
     * Package-level API
     *
     */

    var defaultLogger = new Logger();

    var _loggersByName = {};
    defaultLogger.getLogger = function getLogger(name) {
        if (typeof name !== "string" || name === "") {
          throw new TypeError("You must supply a name when creating a logger.");
        }

        var logger = _loggersByName[name];
        if (!logger) {
          logger = _loggersByName[name] = new Logger(
            name, defaultLogger.getLevel(), defaultLogger.methodFactory);
        }
        return logger;
    };

    // Grab the current global log variable in case of overwrite
    var _log = (typeof window !== undefinedType) ? window.log : undefined;
    defaultLogger.noConflict = function() {
        if (typeof window !== undefinedType &&
               window.log === defaultLogger) {
            window.log = _log;
        }

        return defaultLogger;
    };

    return defaultLogger;
}));
;
/**@license MIT-promiscuous-Â©Ruben Verborgh*/
!function(n,t){function c(n,t){return(typeof t)[0]==n}function u(o,e){return e=function f(i,h,l,a,p,s){function y(n){return function(t){p&&(p=0,f(c,n,t))}}if(a=f.q,i!=c)return u(function(n,t){a.push({p:this,r:n,j:t,1:i,0:h})});if(l&&c(n,l)|c(t,l))try{p=l.then}catch(j){h=0,l=j}if(c(n,p))try{p.call(l,y(1),h=y(0))}catch(j){h(j)}else for(e=function(t,e){return c(n,t=h?t:e)?u(function(n,c){r(this,n,c,l,t)}):o},s=0;s<a.length;)p=a[s++],c(n,i=p[h])?r(p.p,p.r,p.j,l,i):(h?p.r:p.j)(l)},e.q=[],o.call(o={then:function(n,t){return e(n,t)},"catch":function(n){return e(0,n)}},function(n){e(c,1,n)},function(n){e(c,0,n)}),o}function r(u,r,o,e,f){setTimeout(function(){try{e=f(e),f=e&&c(t,e)|c(n,e)&&e.then,c(n,f)?e==u?o(TypeError()):f.call(e,r,o):r(e)}catch(i){o(i)}})}function o(n){return u(function(t){t(n)})}Promise=u,u.resolve=o,u.reject=function(n){return u(function(t,c){c(n)})},u.all=function(n){return u(function(t,c,u,r){r=[],u=n.length||t(r),n.map(function(n,e){o(n).then(function(n){r[e]=n,--u||t(r)},c)})})}}("f","o");;
(function(){var m=new function(){function g(a){return a?0:-1}var e=this.priority=function(a,b){for(var c=a.exprs,f=0,d=0,e=c.length;d<e;d++){var h=c[d];if(!~(h=h.e(h.v,b instanceof Date?b.getTime():b,b)))return-1;f+=h}return f},d=this.parse=function(a,b){a||(a={$eq:a});var c=[];if(a.constructor==Object)for(var f in a){var g=l[f]?f:"$trav",k=a[f],h=k;if(j[g]){if(~f.indexOf(".")){h=f.split(".");f=h.shift();for(var n={},m=n,p=0,s=h.length-1;p<s;p++)m=m[h[p]]={};m[h[p]]=k;h=k=n}if(k instanceof Array){h=
[];for(n=k.length;n--;)h.push(d(k[n]))}else h=d(k,f)}c.push(r(g,f,h))}else c.push(r("$eq",f,a));var q={exprs:c,k:b,test:function(a){return!!~q.priority(a)},priority:function(a){return e(q,a)}};return q},j=this.traversable={$and:!0,$or:!0,$nor:!0,$trav:!0,$not:!0},l=this.testers={$eq:function(a,b){return g(a.test(b))},$ne:function(a,b){return g(!a.test(b))},$lt:function(a,b){return a>b?0:-1},$gt:function(a,b){return a<b?0:-1},$lte:function(a,b){return a>=b?0:-1},$gte:function(a,b){return a<=b?0:-1},
$exists:function(a,b){return a===(null!=b)?0:-1},$in:function(a,b){if(b instanceof Array)for(var c=b.length;c--;){if(~a.indexOf(b[c]))return c}else return g(~a.indexOf(b));return-1},$not:function(a,b){if(!a.test)throw Error("$not test should include an expression, not a value. Use $ne instead.");return g(!a.test(b))},$type:function(a,b,c){return c?c instanceof a||c.constructor==a?0:-1:-1},$nin:function(a,b){return~l.$in(a,b)?-1:0},$mod:function(a,b){return b%a[0]==a[1]?0:-1},$all:function(a,b){for(var c=
a.length;c--;)if(!~b.indexOf(a[c]))return-1;return 0},$size:function(a,b){return b?a==b.length?0:-1:-1},$or:function(a,b){for(var c=a.length,f=c;c--;)if(~e(a[c],b))return c;return 0==f?0:-1},$nor:function(a,b){for(var c=a.length;c--;)if(~e(a[c],b))return-1;return 0},$and:function(a,b){for(var c=a.length;c--;)if(!~e(a[c],b))return-1;return 0},$trav:function(a,b){if(b instanceof Array){for(var c=b.length;c--;){var f=b[c];if(f[a.k]&&~e(a,f[a.k]))return c}return-1}return e(a,b?b[a.k]:void 0)},$regex:function(a,
b){return RegExp(a).test(b)?0:-1}},k={$eq:function(a){return a instanceof RegExp?a:{test:a instanceof Function?a:function(b){return b instanceof Array?~b.indexOf(a):a==b}}},$ne:function(a){return k.$eq(a)}},r=function(a,b,c){c=c instanceof Date?c.getTime():c;return{k:b,v:k[a]?k[a](c):c,e:l[a]}}},j=function(g,e,d){"object"!=typeof e&&(d=e,e=void 0);if(d){if("function"!=typeof d)throw Error("Unknown sift selector "+d);}else d=function(d){return d};var j=d,l=m.parse(g);d=function(d){for(var e=[],a,b,
c,f=0,g=d.length;f<g;f++)b=d[f],a=j(b),~(c=l.priority(a))&&e.push({value:b,priority:c});e.sort(function(a,b){return a.priority>b.priority?-1:1});d=Array(e.length);for(f=e.length;f--;)d[f]=e[f].value;return d};d.test=l.test;d.score=l.priority;d.query=g;return e?d(e):d};j.use=function(g){g.operators&&j.useOperators(g.operators)};j.useOperators=function(g){for(var e in g)j.useOperator(e,g[e])};j.useOperator=function(g,e){var d={},d="object"==typeof e?e:{test:e},j="$"+g;m.testers[j]=d.test;if(d.traversable||
d.traverse)m.traversable[j]=!0};"undefined"!=typeof module&&"undefined"!=typeof module.exports?module.exports=j:"undefined"!=typeof window&&(window.sift=j)})();
;
/* global define, Promise */
(function (root, factory) {
    'use strict';
    if (typeof exports === 'object') {
        // CommonJS
        module.exports = factory();
    } else if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define(factory);
    } else {
        // Browser globals
        root.Queue = factory();
    }
})
(this, function () {
    'use strict';

    /**
     * @return {Object}
     */
    var LocalPromise = typeof Promise !== 'undefined' ? Promise : function () {
        return {
            then: function () {
                throw new Error('Queue.configure() before use Queue');
            }
        };
    };

    var noop = function () {};

    /**
     * @param {*} value
     * @returns {LocalPromise}
     */
    var resolveWith = function (value) {
        if (value && typeof value.then === 'function') {
            return value;
        }

        return new LocalPromise(function (resolve) {
            resolve(value);
        });
    };

    /**
     * It limits concurrently executed promises
     *
     * @param {Number} [maxPendingPromises=Infinity] max number of concurrently executed promises
     * @param {Number} [maxQueuedPromises=Infinity]  max number of queued promises
     * @constructor
     *
     * @example
     *
     * var queue = new Queue(1);
     *
     * queue.add(function () {
     *     // resolve of this promise will resume next request
     *     return downloadTarballFromGithub(url, file);
     * })
     * .then(function (file) {
     *     doStuffWith(file);
     * });
     *
     * queue.add(function () {
     *     return downloadTarballFromGithub(url, file);
     * })
     * // This request will be paused
     * .then(function (file) {
     *     doStuffWith(file);
     * });
     */
    function Queue(maxPendingPromises, maxQueuedPromises) {
        this.pendingPromises = 0;
        this.maxPendingPromises = typeof maxPendingPromises !== 'undefined' ? maxPendingPromises : Infinity;
        this.maxQueuedPromises = typeof maxQueuedPromises !== 'undefined' ? maxQueuedPromises : Infinity;
        this.queue = [];
    }

    /**
     * Defines promise promiseFactory
     * @param {Function} GlobalPromise
     */
    Queue.configure = function (GlobalPromise) {
        LocalPromise = GlobalPromise;
    };

    /**
     * @param {Function} promiseGenerator
     * @return {LocalPromise}
     */
    Queue.prototype.add = function (promiseGenerator) {
        var self = this;
        return new LocalPromise(function (resolve, reject, notify) {
            // Do not queue to much promises
            if (self.queue.length >= self.maxQueuedPromises) {
                reject(new Error('Queue limit reached'));
                return;
            }

            // Add to queue
            self.queue.push({
                promiseGenerator: promiseGenerator,
                resolve: resolve,
                reject: reject,
                notify: notify || noop
            });

            self._dequeue();
        });
    };

    /**
     * Number of simultaneously running promises (which are resolving)
     *
     * @return {number}
     */
    Queue.prototype.getPendingLength = function () {
        return this.pendingPromises;
    };

    /**
     * Number of queued promises (which are waiting)
     *
     * @return {number}
     */
    Queue.prototype.getQueueLength = function () {
        return this.queue.length;
    };

    /**
     * @returns {boolean} true if first item removed from queue
     * @private
     */
    Queue.prototype._dequeue = function () {
        var self = this;

        if (this.pendingPromises >= this.maxPendingPromises) {
            return false;
        }

        // Remove from queue
        var item = this.queue.shift();
        if (!item) {
            return false;
        }

        this.pendingPromises++;
        resolveWith(item.promiseGenerator())
            // Forward all stuff
            .then(function (value) {
                // It is not pending now
                self.pendingPromises--;
                self._dequeue();
                // It should pass values
                item.resolve(value);
            }, function (err) {
                // It is not pending now
                self.pendingPromises--;
                self._dequeue();
                // It should not mask errors
                item.reject(err);
            }, function (message) {
                // It should pass notifications
                item.notify(message);
            });

        return true;
    };

    return Queue;
});
;
/*!
 * Copyright (c) 2015 Kinvey, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function(){var a=this,b=function(){function c(){"undefined"==typeof openDatabase&&"undefined"==typeof a.openDatabase||"undefined"==typeof a.sift||"undefined"==typeof a.Queue||(a.openDatabase="undefined"!=typeof openDatabase?openDatabase:a.openDatabase,X.use($),["near","regex","within"].forEach(function(b){a.sift.useOperator(b,function(){throw new e.Error(b+" query operator is not supported locally.")})}))}function d(){"undefined"!=typeof _.impl&&"undefined"!=typeof a.sift&&(X.use(_),["near","regex","within"].forEach(function(b){a.sift.useOperator(b,function(){throw new e.Error(b+" query operator is not supported locally.")})}))}var e=function(a){var c=b();return null!=a&&c.init(a),c},f={};if("undefined"!=typeof a.log){f=a.log.noConflict();var g=f.methodFactory;f.methodFactory=function(a,b){var c=g(a,b);return function(a,b){a="Kinvey: "+a,c(a,b)}}}e.Log={levels:f.levels,getLevel:function(){return f.getLevel()},setLevel:function(a,b){f.setLevel(a,b)},setDefaultLevel:function(a){f.setDefaultLevel(a)},enableAll:function(a){f.enableAll(a)},disableAll:function(a){f.disableAll(a)}},e.Log.setDefaultLevel(e.Log.levels.ERROR),e.APIHostName="https://baas.kinvey.com",e.API_ENDPOINT=void 0,e.MICHostName="https://auth.kinvey.com",e.MICAPIVersion=void 0,e.API_VERSION="3",e.SDK_VERSION="1.5.1",e.appKey=null,e.appSecret=null,e.masterSecret=null;var h="appdata",i="blob",j="rpc",k="user",l=2e3,m=null,n=!1,o=function(a){var b=F.get("activeUser");return b.then(function(b){if(null==b)return e.setActiveUser(null);f.debug("Restoring the active user.");var c=e.setActiveUser({_id:b[0],_kmd:{authtoken:b[1]}});if(!1===a.refresh)return e.getActiveUser();var d=a.success,g=a.error;return delete a.success,delete a.error,e.User.me(a).then(function(b){return f.debug("Restored the active user.",b),a.success=d,a.error=g,b},function(b){return f.error("Failed to restore the active user.",b),e.Error.INVALID_CREDENTIALS===b.name&&e.setActiveUser(c),a.success=d,a.error=g,e.Defer.reject(b)})})};e.getActiveUser=function(){if(!1===n)throw new e.Error("Kinvey.getActiveUser can only be called after the promise returned by Kinvey.init fulfills or rejects.");return m},e.setActiveUser=function(a){if(f.debug("Setting the active user.",arguments),null!=a&&(null==a._id||null==a._kmd||null==a._kmd.authtoken))throw new e.Error("user argument must contain: _id, _kmd.authtoken.");!1===n&&(n=!0);var b=e.getActiveUser();return m=a,null!=a?F.save("activeUser",[a._id,a._kmd.authtoken]):F.destroy("activeUser"),b},e.init=function(a){var b;if(f.debug("Initializing the copy of the library.",arguments),a=a||{},null==a.appKey)return b=new e.Error("options argument must contain: appKey."),t(e.Defer.reject(b),a);if(null==a.appSecret&&null==a.masterSecret)return b=new e.Error("options argument must contain: appSecret and/or masterSecret."),t(e.Defer.reject(b),a);n=!1;var c=a.apiHostName||e.API_ENDPOINT;if(e.APIHostName=c||e.APIHostName,0!==e.APIHostName.indexOf("https://"))return b=new e.Error("Kinvey requires https as the protocol when setting Kinvey.APIHostName, instead found the protocol "+e.APIHostName.substring(0,e.APIHostName.indexOf(":/"))+" in Kinvey.APIHostName: "+e.APIHostName),t(e.Defer.reject(b),a);if(e.MICHostName=a.micHostName||e.MICHostName,0!==e.MICHostName.indexOf("https://"))return b=new e.Error("Kinvey requires https as the protocol when setting Kinvey.MICHostName, instead found the protocol "+e.MICHostName.substring(0,e.MICHostName.indexOf(":/"))+" in Kinvey.MICHostName: "+e.MICHostName),t(e.Defer.reject(b),a);e.MICAPIVersion=a.micApiVersion||e.MICAPIVersion,null!=a.clientAppVersion&&e.ClientAppVersion.setVersion(a.clientAppVersion),null!=a.customRequestProperties&&e.CustomRequestProperties.setProperties(a.customRequestProperties),e.appKey=a.appKey,e.appSecret=null!=a.appSecret?a.appSecret:null,e.masterSecret=null!=a.masterSecret?a.masterSecret:null,e.encryptionKey=null!=a.encryptionKey?a.encryptionKey:null;var d=X.upgrade().then(function(){return e.Sync.init(a.sync)}).then(function(){return f.debug("Kinvey initialized, running version: js-phonegap/1.5.1"),o(a)});return t(d,a)},e.ping=function(a){f.debug("Pinging the Kinvey service.",arguments),a=a||{},a.nocache=null==e.appKey?!1:a.nocache;var b=e.Persistence.read({namespace:h,auth:null!=e.appKey?H.All:H.None},a);return b.then(function(a){f.debug("Pinged the Kinvey service.",a)},function(a){f.error("Failed to ping the Kinvey service.",a)}),t(b,a)},e.Error=function(a){this.name="Kinvey.Error",this.message=a,this.stack=(new Error).stack,f.error("A Kinvey.Error was thrown.",this.message,this.stack)},e.Error.prototype=new Error,e.Error.prototype.constructor=e.Error,e.Error.ENTITY_NOT_FOUND="EntityNotFound",e.Error.COLLECTION_NOT_FOUND="CollectionNotFound",e.Error.APP_NOT_FOUND="AppNotFound",e.Error.USER_NOT_FOUND="UserNotFound",e.Error.BLOB_NOT_FOUND="BlobNotFound",e.Error.INVALID_CREDENTIALS="InvalidCredentials",e.Error.KINVEY_INTERNAL_ERROR_RETRY="KinveyInternalErrorRetry",e.Error.KINVEY_INTERNAL_ERROR_STOP="KinveyInternalErrorStop",e.Error.USER_ALREADY_EXISTS="UserAlreadyExists",e.Error.USER_UNAVAILABLE="UserUnavailable",e.Error.DUPLICATE_END_USERS="DuplicateEndUsers",e.Error.INSUFFICIENT_CREDENTIALS="InsufficientCredentials",e.Error.WRITES_TO_COLLECTION_DISALLOWED="WritesToCollectionDisallowed",e.Error.INDIRECT_COLLECTION_ACCESS_DISALLOWED="IndirectCollectionAccessDisallowed",e.Error.APP_PROBLEM="AppProblem",e.Error.PARAMETER_VALUE_OUT_OF_RANGE="ParameterValueOutOfRange",e.Error.CORS_DISABLED="CORSDisabled",e.Error.INVALID_QUERY_SYNTAX="InvalidQuerySyntax",e.Error.MISSING_QUERY="MissingQuery",e.Error.JSON_PARSE_ERROR="JSONParseError",e.Error.MISSING_REQUEST_HEADER="MissingRequestHeader",e.Error.INCOMPLETE_REQUEST_BODY="IncompleteRequestBody",e.Error.MISSING_REQUEST_PARAMETER="MissingRequestParameter",e.Error.INVALID_IDENTIFIER="InvalidIdentifier",e.Error.BAD_REQUEST="BadRequest",e.Error.FEATURE_UNAVAILABLE="FeatureUnavailable",e.Error.API_VERSION_NOT_IMPLEMENTED="APIVersionNotImplemented",e.Error.API_VERSION_NOT_AVAILABLE="APIVersionNotAvailable",e.Error.INPUT_VALIDATION_FAILED="InputValidationFailed",e.Error.BL_RUNTIME_ERROR="BLRuntimeError",e.Error.BL_SYNTAX_ERROR="BLSyntaxError",e.Error.BL_TIMEOUT_ERROR="BLTimeoutError",e.Error.OAUTH_TOKEN_REFRESH_ERROR="OAuthTokenRefreshError",e.Error.BL_VIOLATION_ERROR="BLViolationError",e.Error.BL_INTERNAL_ERROR="BLInternalError",e.Error.THIRD_PARTY_TOS_UNACKED="ThirdPartyTOSUnacked",e.Error.STALE_REQUEST="StaleRequest",e.Error.DATA_LINK_PARSE_ERROR="DataLinkParseError",e.Error.NOT_IMPLEMENTED_ERROR="NotImplementedError",e.Error.EMAIL_VERIFICATION_REQUIRED="EmailVerificationRequired",e.Error.SORT_LIMIT_EXCEEDED="SortLimitExceeded",e.Error.INVALID_SHORT_URL="InvalidShortURL",e.Error.INVALID_OR_MISSING_NONCE="InvalidOrMissingNonce",e.Error.MISSING_CONFIGURATION="MissingConfiguration",e.Error.ENDPOINT_DOES_NOT_EXIST="EndpointDoesNotExist",e.Error.DISALLOWED_QUERY_SYNTAX="DisallowedQuerySyntax",e.Error.MALFORMED_AUTHENTICATION_HEADER="MalformedAuthenticationHeader",e.Error.APP_ARCHIVED="AppArchived",e.Error.BL_NOT_SUPPORTED_FOR_ROUTE="BLNotSupportedForRoute",e.Error.USER_LOCKED_DOWN="UserLockedDown",e.Error.ALREADY_LOGGED_IN="AlreadyLoggedIn",e.Error.DATABASE_ERROR="DatabaseError",e.Error.MISSING_APP_CREDENTIALS="MissingAppCredentials",e.Error.MISSING_MASTER_CREDENTIALS="MissingMasterCredentials",e.Error.NO_ACTIVE_USER="NoActiveUser",e.Error.REQUEST_ABORT_ERROR="RequestAbortError",e.Error.REQUEST_ERROR="RequestError",e.Error.REQUEST_TIMEOUT_ERROR="RequestTimeoutError",e.Error.SOCIAL_ERROR="SocialError",e.Error.SYNC_ERROR="SyncError",e.Error.MIC_ERROR="MIC_ERROR";var p={};p[e.Error.ALREADY_LOGGED_IN]={name:e.Error.ALREADY_LOGGED_IN,description:"You are already logged in with another user.",debug:"If you want to switch users, logout the active user first using `Kinvey.User.logout`, then try again."},p[e.Error.DATABASE_ERROR]={name:e.Error.DATABASE_ERROR,description:"The database used for local persistence encountered an error.",debug:""},p[e.Error.MISSING_APP_CREDENTIALS]={name:e.Error.MISSING_APP_CREDENTIALS,description:"Missing credentials: `Kinvey.appKey` and/or `Kinvey.appSecret`.",debug:"Did you forget to call `Kinvey.init`?"},p[e.Error.MISSING_MASTER_CREDENTIALS]={name:e.Error.MISSING_MASTER_CREDENTIALS,description:"Missing credentials: `Kinvey.appKey` and/or `Kinvey.masterSecret`.",debug:"Did you forget to call `Kinvey.init` with your Master Secret?"},p[e.Error.NO_ACTIVE_USER]={name:e.Error.NO_ACTIVE_USER,description:"You need to be logged in to execute this request.",debug:"Try creating a user using `Kinvey.User.signup`, or login an existing user using `Kinvey.User.login`."},p[e.Error.REQUEST_ABORT_ERROR]={name:e.Error.REQUEST_TIMEOUT_ERROR,description:"The request was aborted.",debug:""},p[e.Error.REQUEST_ERROR]={name:e.Error.REQUEST_ERROR,description:"The request failed.",debug:""},p[e.Error.REQUEST_TIMEOUT_ERROR]={name:e.Error.REQUEST_TIMEOUT_ERROR,description:"The request timed out.",debug:""},p[e.Error.SOCIAL_ERROR]={name:e.Error.SOCIAL_ERROR,description:"The social identity cannot be obtained.",debug:""},p[e.Error.SYNC_ERROR]={name:e.Error.SYNC_ERROR,description:"The synchronization operation cannot be completed.",debug:""},p[e.Error.MIC_ERROR]={name:e.Error.MIC_ERROR,description:"Unable to authorize using Mobile Identity Connect.",debug:""};var q,r=function(a,b){var c=p[a]||{name:a};return b=b||{},{name:c.name,description:b.description||c.description||"",debug:b.debug||c.debug||""}},s=function(a,b,c){if(!b)return a="undefined"==typeof c?a:c;for(var d,e=a,f=b.split(".");(d=f.shift())&&null!=e&&e.hasOwnProperty(d);){if(0===f.length)return e[d]="undefined"==typeof c?e[d]:c,e[d];e=e[d]}return null};q="function"==typeof a.setImmediate?a.setImmediate:"undefined"!=typeof process&&process.nextTick?process.nextTick:function(b){a.setTimeout(b,0)};var t=function(a,b){return a.then(function(a){b.success&&b.success(a)},function(a){b.error&&b.error(a)}).then(null,function(a){q(function(){throw a})}),a},u=Array.isArray||function(a){return"[object Array]"===Object.prototype.toString.call(a)},v=function(a){return"function"!=typeof/./?"function"==typeof a:"[object Function]"===Object.prototype.toString.call(a)},w=function(a){return"[object Number]"===Object.prototype.toString.call(a)&&!isNaN(a)},x=function(a){return Object(a)===a},y=function(a){return"[object RegExp]"===Object.prototype.toString.call(a)},z=function(a){return"[object String]"===Object.prototype.toString.call(a)},A=function(a){if(null==a)return!0;if(u(a)||z(a))return 0===a.length;for(var b in a)if(a.hasOwnProperty(b))return!1;return!0},B=function(a){return function(){throw new e.Error("Method not implemented: "+a)}},C=function(a){return function(b){var c=this;f.debug("Applying an adapter.",c,b),b=b||{},a.forEach(function(a){if("function"!=typeof b[a])throw new e.Error("Adapter must implement method: "+a)}),a.forEach(function(a){c[a]=function(){return b[a].apply(b,arguments)}})}},D=function(a){var b,c=0,d=a.length;for(a=String(a||""),b=0;d>b;b++){var e=encodeURI(a[b]).split("%").length;c+=1===e?1:e-1}return c},E=function(a){if("string"!=typeof a)return{};if(a=a.trim().replace(/^(\?|#)/,""),!a)return{};var b=a.indexOf("#/");return b===a.length-2&&(a=a.substring(0,b)),a.trim().split("&").reduce(function(a,b){var c=b.replace(/\+/g," ").split("="),d=c[0],e=c[1];return d=decodeURIComponent(d),e=void 0===e?null:decodeURIComponent(e),a.hasOwnProperty(d)?Array.isArray(a[d])?a[d].push(e):a[d]=[a[d],e]:a[d]=e,a},{})},F={destroy:function(a){return F._destroy(F._key(a))},get:function(a){return F._get(F._key(a))},save:function(a,b){return F._save(F._key(a),b)},_destroy:B("Storage.destroy"),_get:B("Storage.get"),_key:function(a){return["Kinvey",e.appKey,a].join(".")},_save:B("Storage.set"),use:C(["_destroy","_get","_save"])};e.Defer={all:function(a){var b;if(!u(a))return b=new e.Error("promises argument must be of type: Array."),e.Defer.reject(b);var c=a.length;if(0===c)return e.Defer.resolve([]);var d=e.Defer.deferred(),f=[];return a.forEach(function(a,b){a.then(function(a){c-=1,f[b]=a,0===c&&d.resolve(f)},function(a){d.reject(a)})}),d.promise},resolve:function(a){var b=e.Defer.deferred();return b.resolve(a),b.promise},reject:function(a){var b=e.Defer.deferred();return b.reject(a),b.promise},deferred:B("Kinvey.Defer.deferred"),use:C(["deferred"])};var G,H={All:function(){return H.Session().then(null,H.Basic)},App:function(){if(null==e.appKey||null==e.appSecret){var a=r(e.Error.MISSING_APP_CREDENTIALS);return e.Defer.reject(a)}var b=e.Defer.resolve({scheme:"Basic",username:e.appKey,password:e.appSecret});return b.then(function(a){f.debug("Authenticating through App Secret.",a)}),b},Basic:function(){return H.Master().then(null,H.App)},Default:function(){return H.Session().then(null,function(a){return H.Master().then(null,function(){return e.Defer.reject(a)})})},Master:function(){if(null==e.appKey||null==e.masterSecret){var a=r(e.Error.MISSING_MASTER_CREDENTIALS);return e.Defer.reject(a)}var b=e.Defer.resolve({scheme:"Basic",username:e.appKey,password:e.masterSecret});return b.then(function(a){f.debug("Authenticating through Master Secret.",a)}),b},None:function(){return e.Defer.resolve(null)},Session:function(){var a,b=e.getActiveUser();if(null===b)return a=r(e.Error.NO_ACTIVE_USER),e.Defer.reject(a);if(null===b._kmd||void 0===b._kmd)return a=new e.Error("The active user does not have _kmd defined as a property.It is required to authenticate the user. User _id: "+b._id),e.Defer.reject(a);var c=e.Defer.resolve({scheme:"Kinvey",credentials:b._kmd.authtoken});return c.then(function(a){f.debug("Authenticating through user credentials.",a)}),c}},I=function(){var b,c,d,e,f,g=[],h=function(a){a=a.toLowerCase();var b=/(chrome)\/([\w]+)/,c=/(firefox)\/([\w.]+)/,d=/(msie) ([\w.]+)/i,e=/(opera)(?:.*version)?[ \/]([\w.]+)/,f=/(safari)\/([\w.]+)/;return b.exec(a)||c.exec(a)||d.exec(a)||e.exec(a)||f.exec(a)||[]};if("undefined"!=typeof a.cordova&&"undefined"!=typeof a.device){var i=a.device;g.push("phonegap/"+i.cordova),c=i.platform,d=i.version,e=i.model,f=i.uuid}else"undefined"!=typeof Titanium?(g.push("titanium/"+Titanium.getVersion()),"mobileweb"===Titanium.Platform.getName()?(b=h(Titanium.Platform.getModel()),c=b[1],d=b[2],e=Titanium.Platform.getOstype()):(c=Titanium.Platform.getOsname(),d=Titanium.Platform.getVersion(),e=Titanium.Platform.getManufacturer()),f=Titanium.Platform.getId()):"undefined"!=typeof forge?(g.push("triggerio/"+(forge.config.platform_version||"")),f=forge.config.uuid):"undefined"!=typeof process&&(c=process.title,d=process.version,e=process.platform);"undefined"!=typeof angular&&g.push("angularjs/"+angular.version.full),"undefined"!=typeof Backbone&&g.push("backbonejs/"+Backbone.VERSION),"undefined"!=typeof Ember&&g.push("emberjs/"+Ember.VERSION),"undefined"!=typeof jQuery&&g.push("jquery/"+jQuery.fn.jquery),"undefined"!=typeof ko&&g.push("knockout/"+ko.version),"undefined"!=typeof Zepto&&g.push("zeptojs"),null==c&&a.navigator&&(b=h(a.navigator.userAgent),c=b[1],d=b[2],e=a.navigator.platform);var j=["js-phonegap/1.5.1"];return 0!==g.length&&j.push("("+g.sort().join(", ")+")"),j.concat([c,d,e,f].map(function(a){return null!=a?a.toString().replace(/\s/g,"_").toLowerCase():"unknown"})).join(" ")},J=function(){var a=arguments[0];if(arguments.length>1){var b=arguments[0],c=arguments[1],d=arguments[2];a=(b+"").trim(),null!=c&&(a+=("."+c).trim()),null!=d&&(a+=("."+d).trim())}return a},K=function(a){return null==a?void 0:(a+"").trim()},L=function(){G=void 0};e.ClientAppVersion={stringValue:function(){return K(G)},setVersion:function(){e.ClientAppVersion.clear(),f.debug("Setting the client app version.",arguments),G=J.apply(a,arguments)},clear:function(){f.debug("Clearing the client app version."),L()}};var M={},N=function(a){null!=a&&M.hasOwnProperty(a)&&delete M[a]},O=function(){M={}};e.CustomRequestProperties={properties:function(){return JSON.parse(JSON.stringify(M))},property:function(a){return null!=a&&M.hasOwnProperty(a)?M[a]:void 0},setProperties:function(a){e.CustomRequestProperties.clear(),e.CustomRequestProperties.addProperties(a)},setProperty:function(a,b){var c={};c[a]=b,e.CustomRequestProperties.addProperties(c)},addProperties:function(a){null!=a&&Object.keys(a).forEach(function(b){var c=a[b];f.debug("Adding custom request property "+b+" as "+c+"."),M[b]=c})},clear:function(){f.debug("Clearing the custom request properties."),O()},clearProperty:function(a){f.debug("Clearing the custom request property "+a+"."),N(a)}},e.Acl=function(a){if(null!=a&&!x(a))throw new e.Error("document argument must be of type: Object.");a=a||{},a._acl=a._acl||{},this._acl=a._acl},e.Acl.prototype={addReader:function(a){return this._acl.r=this._acl.r||[],-1===this._acl.r.indexOf(a)&&this._acl.r.push(a),this},addReaderGroup:function(a){return this._acl.groups=this._acl.groups||{},this._acl.groups.r=this._acl.groups.r||[],-1===this._acl.groups.r.indexOf(a)&&this._acl.groups.r.push(a),this},addWriterGroup:function(a){return this._acl.groups=this._acl.groups||{},this._acl.groups.w=this._acl.groups.w||[],-1===this._acl.groups.w.indexOf(a)&&this._acl.groups.w.push(a),this},addWriter:function(a){return this._acl.w=this._acl.w||[],-1===this._acl.w.indexOf(a)&&this._acl.w.push(a),this},getCreator:function(){return this._acl.creator||null},getReaders:function(){return this._acl.r||[]},getReaderGroups:function(){return this._acl.groups?this._acl.groups.r:[]},getWriterGroups:function(){return this._acl.groups?this._acl.groups.w:[]},getWriters:function(){return this._acl.w||[]},isGloballyReadable:function(){return this._acl.gr||!1},isGloballyWritable:function(){return this._acl.gw||!1},removeReader:function(a){var b;return this._acl.r&&-1!==(b=this._acl.r.indexOf(a))&&this._acl.r.splice(b,1),this},removeReaderGroup:function(a){var b;return this._acl.groups&&this._acl.groups.r&&-1!==(b=this._acl.groups.r.indexOf(a))&&this._acl.groups.r.splice(b,1),this},removeWriterGroup:function(a){var b;return this._acl.groups&&this._acl.groups.w&&-1!==(b=this._acl.groups.w.indexOf(a))&&this._acl.groups.w.splice(b,1),this},removeWriter:function(a){var b;return this._acl.w&&-1!==(b=this._acl.w.indexOf(a))&&this._acl.w.splice(b,1),this},setGloballyReadable:function(a){return this._acl.gr=a||!1,this},setGloballyWritable:function(a){return this._acl.gw=a||!1,this},toJSON:function(){return this._acl}},e.Group=function(){this._query=null,this._initial={},this._key={},this._reduce=function(){}.toString()},e.Group.prototype={by:function(a){return this._key[a]=!0,this},initial:function(a,b){if("undefined"==typeof b&&!x(a))throw new e.Error("objectOrKey argument must be of type: Object.");return x(a)?this._initial=a:this._initial[a]=b,this},postProcess:function(a){return null===this._query?a:this._query._postProcess(a)},query:function(a){if(!(a instanceof e.Query))throw new e.Error("query argument must be of type: Kinvey.Query.");return this._query=a,this},reduce:function(a){if(v(a)&&(a=a.toString()),!z(a))throw new e.Error("fn argument must be of type: function or string.");return this._reduce=a,this},toJSON:function(){return{key:this._key,initial:this._initial,reduce:this._reduce,condition:null!==this._query?this._query.toJSON().filter:{}}}},e.Group.count=function(a){var b=new e.Group;return null!=a&&b.by(a),b.initial({result:0}),b.reduce(function(a,b){b.result+=1}),b},e.Group.sum=function(a){a=a.replace("'","\\'");var b=new e.Group;return b.initial({result:0}),b.reduce('function(doc, out) { out.result += doc["'+a+'"]; }'),b},e.Group.min=function(a){a=a.replace("'","\\'");var b=new e.Group;return b.initial({result:"Infinity"}),b.reduce('function(doc, out) { out.result = Math.min(out.result, doc["'+a+'"]); }'),b},e.Group.max=function(a){a=a.replace("'","\\'");var b=new e.Group;return b.initial({result:"-Infinity"}),b.reduce('function(doc, out) { out.result = Math.max(out.result, doc["'+a+'"]); }'),b},e.Group.average=function(a){a=a.replace("'","\\'");var b=new e.Group;return b.initial({count:0,result:0}),b.reduce('function(doc, out) {  out.result = (out.result * out.count + doc["'+a+'"]) / (out.count + 1);  out.count += 1;}'),b},e.execute=function(a,b,c){f.debug("Executing custom command.",arguments),c=c||{};var d=e.Persistence.create({namespace:j,collection:"custom",id:a,data:b,auth:H.Default},c).then(null,function(a){return e.Defer.reject(e.Error.REQUEST_ERROR===a.name&&x(a.debug)?a.debug:a)});return d.then(function(a){f.debug("Executed the custom command.",a)},function(a){f.error("Failed to execute the custom command.",a)}),t(d,c)},e.DataStore={find:function(a,b,c){var d;if(f.debug("Retrieving documents by query.",arguments),null!=b&&!(b instanceof e.Query))return d=new e.Error("query argument must be of type: Kinvey.Query."),t(e.Defer.reject(d),c);c=c||{};var g=e.Persistence.read({namespace:h,collection:a,query:b,auth:H.Default,local:{req:!0,res:!0}},c);return g.then(function(a){f.debug("Retrieved the documents by query.",a)},function(a){f.error("Failed to retrieve the documents by query.",a)}),t(g,c)},get:function(a,b,c){f.debug("Retrieving a document.",arguments),c=c||{};var d=e.Persistence.read({namespace:h,collection:a,id:b,auth:H.Default,local:{req:!0,res:!0}},c);return d.then(function(a){f.debug("Retrieved the document.",a)},function(a){f.error("Failed to retrieve the document.",a)}),t(d,c)},save:function(a,b,c){if(f.debug("Saving a (new) document.",arguments),c=c||{},null!=b._id)return f.debug("The document has an _id, updating instead.",arguments),e.DataStore.update(a,b,c);var d=e.Persistence.create({namespace:h,collection:a,data:b,auth:H.Default,local:{req:!0,res:!0}},c);return d.then(function(a){f.debug("Saved the new document.",a)},function(a){f.error("Failed to save the new document.",a)}),t(d,c)},update:function(a,b,c){var d;if(f.debug("Updating a document.",arguments),null==b._id)return d=new e.Error("document argument must contain: _id"),t(e.Defer.reject(d),c);c=c||{};var g=e.Persistence.update({namespace:h,collection:a,id:b._id,data:b,auth:H.Default,local:{req:!0,res:!0}},c);return g.then(function(a){f.debug("Updated the document.",a)},function(a){f.error("Failed to update the document.",a)}),t(g,c)},clean:function(a,b,c){var d;if(f.debug("Deleting documents by query.",arguments),c=c||{},b=b||new e.Query,!(b instanceof e.Query))return d=new e.Error("query argument must be of type: Kinvey.Query."),t(e.Defer.reject(d),c);var g=e.Persistence.destroy({namespace:h,collection:a,query:b,auth:H.Default,local:{req:!0,res:!0}},c);return g.then(function(a){f.debug("Deleted the documents.",a)},function(a){f.error("Failed to delete the documents.",a)}),t(g,c)},destroy:function(a,b,c){f.debug("Deleting a document.",arguments),c=c||{};var d=e.Persistence.destroy({namespace:h,collection:a,id:b,auth:H.Default,local:{req:!0,res:!0}},c).then(null,function(a){return c.silent&&e.Error.ENTITY_NOT_FOUND===a.name?(f.debug("The document does not exist. Returning success because of the silent flag."),{count:0}):e.Defer.reject(a)});return d.then(function(a){f.debug("Deleted the document.",a)},function(a){f.error("Failed to delete the document.",a)}),t(d,c)},count:function(a,b,c){var d;if(f.debug("Counting the number of documents.",arguments),null!=b&&!(b instanceof e.Query))return d=new e.Error("query argument must be of type: Kinvey.Query."),t(e.Defer.reject(d),c);c=c||{};var g=e.Persistence.read({namespace:h,collection:a,id:"_count",query:b,auth:H.Default,local:{req:!0}},c).then(function(a){return a.count});return g.then(function(a){f.debug("Counted the number of documents.",a)},function(a){f.error("Failed to count the number of documents.",a)}),t(g,c)},group:function(a,b,c){var d;if(f.debug("Grouping documents",arguments),!(b instanceof e.Group))return d=new e.Error("aggregation argument must be of type: Kinvey.Group."),t(e.Defer.reject(d),c);c=c||{};var g=e.Persistence.create({namespace:h,collection:a,id:"_group",data:b.toJSON(),auth:H.Default,local:{req:!0}},c).then(function(a){return b.postProcess(a)});return g.then(function(a){f.debug("Grouped the documents.",a)},function(a){f.error("Failed to group the documents.",a)}),t(g,c)}},e.File={destroy:function(a,b){f.debug("Deleting a file.",arguments),b=b||{};var c=e.Persistence.destroy({namespace:i,id:a,auth:H.Default},b).then(null,function(a){return b.silent&&e.Error.BLOB_NOT_FOUND===a.name?(f.debug("The file does not exist. Returning success because of the silent flag."),{count:0}):e.Defer.reject(a)});return c.then(function(a){f.debug("Deleted the file.",a)},function(a){f.error("Failed to delete the file.",a)}),t(c,b)},download:function(a,b){f.debug("Downloading a file.",arguments),b=b||{};var c={};!1!==b.tls&&(c.tls=!0),b.ttl&&(c.ttl_in_seconds=b.ttl);var d=e.Persistence.read({namespace:i,id:a,flags:c,auth:H.Default},b).then(function(a){if(b.stream)return f.debug("Returning the file metadata only because of the stream flag."),a;var c=b.success,d=b.error;return delete b.success,delete b.error,e.File.downloadByUrl(a,b).then(function(a){return b.success=c,b.error=d,a},function(a){return b.success=c,b.error=d,e.Defer.reject(a)})});return d.then(function(a){f.debug("Downloaded the file.",a)},function(a){f.error("Failed to download the file.",a)}),t(d,b)},downloadByUrl:function(a,b){f.debug("Downloading a file by URL.",arguments);var c=x(a)?a:{_downloadURL:a};b=b||{},b.file=c.mimeType||"application-octet-stream",b.headers=b.headers||{},delete b.headers["Content-Type"];var d=c._downloadURL,g=e.Persistence.Net.request("GET",d,null,b.headers,b);return g=g.then(function(a){return c._data=a,c},function(a){var b=r(e.Error.REQUEST_ERROR,{description:"This file could not be downloaded from the provided URL.",debug:a});return e.Defer.reject(b)}),g.then(function(a){f.debug("Downloaded the file by URL.",a)},function(a){f.error("Failed to download a file by URL.",a)}),t(g,b)},find:function(a,b){var c;if(f.debug("Retrieving files by query.",arguments),null!=a&&!(a instanceof e.Query))return c=new e.Error("query argument must be of type: Kinvey.Query."),t(e.Defer.reject(c),b);b=b||{};var d={};!1!==b.tls&&(d.tls=!0),b.ttl&&(d.ttl_in_seconds=b.ttl);var g=e.Persistence.read({namespace:i,query:a,flags:d,auth:H.Default},b).then(function(a){if(b.download){f.debug("Obtaining the file resources.",a);var c=b.success,d=b.error;delete b.success,delete b.error;var g=a.map(function(a){return e.File.downloadByUrl(a,b)});return e.Defer.all(g).then(function(a){return b.success=c,b.error=d,a},function(a){return b.success=c,b.error=d,e.Defer.reject(a)})}return a});return g.then(function(a){f.debug("Retrieved the files by query.",a)},function(a){f.error("Failed to retrieve the files by query.",a)}),t(g,b)},stream:function(a,b){return f.debug("Streaming a file.",arguments),b=b||{},b.stream=!0,e.File.download(a,b)},upload:function(a,b,c){f.debug("Uploading a file.",arguments),a=a||{},b=b||{},c=c||{},null!=b._filename||null==a._filename&&null==a.name||(b._filename=a._filename||a.name),null!=b.size||null==a.size&&null==a.length||(b.size=a.size||a.length),b.mimeType=b.mimeType||a.mimeType||a.type||"application/octet-stream",c["public"]&&(b._public=!0),c.contentType=b.mimeType;var d=null!=b._id?e.Persistence.update({namespace:i,id:b._id,data:b,flags:!1!==c.tls?{tls:!0}:null,auth:H.Default},c):e.Persistence.create({namespace:i,data:b,flags:!1!==c.tls?{tls:!0}:null,auth:H.Default},c);return d=d.then(function(b){var d=b._uploadURL,f=b._requiredHeaders||{};f["Content-Type"]=c.contentType,delete b._expiresAt,delete b._requiredHeaders,delete b._uploadURL;var g=e.Persistence.Net.request("PUT",d,a,f,c);return g.then(function(){return b._data=a,b})}),d.then(function(a){f.debug("Uploaded the file.",a)},function(a){f.error("Failed to upload the file.",a)}),t(d,c)}};var P=function(b){var c=Date.parse(b);if(c)return new Date(c);var d=/^(\d{4}\-\d\d\-\d\d([tT][\d:\.]*)?)([zZ]|([+\-])(\d\d):?(\d\d))?$/,e=b.match(d);if(null!=e[1]){var f=e[1].split(/\D/).map(function(b){return a.parseInt(b,10)||0});if(f[1]-=1,f=new Date(Date.UTC.apply(Date,f)),null!=e[5]){var g=a.parseInt(e[5],10)/100*60;g+=null!=e[6]?a.parseInt(e[6],10):0,g*="+"===e[4]?-1:1,g&&f.setUTCMinutes(f.getUTCMinutes()*g)}return f}return 0/0};e.Metadata=function(a){if(!x(a))throw new e.Error("document argument must be of type: Object.");this._acl=null,this._document=a},e.Metadata.prototype={getAcl:function(){return null===this._acl&&(this._acl=new e.Acl(this._document)),this._acl},getCreatedAt:function(){return null!=this._document._kmd&&null!=this._document._kmd.ect?P(this._document._kmd.ect):null},getEmailVerification:function(){return null!=this._document._kmd&&null!=this._document._kmd.emailVerification?this._document._kmd.emailVerification.status:null},getLastModified:function(){return null!=this._document._kmd&&null!=this._document._kmd.lmt?P(this._document._kmd.lmt):null},setAcl:function(a){if(!(a instanceof e.Acl))throw new e.Error("acl argument must be of type: Kinvey.Acl.");return this._acl=null,this._document._acl=a.toJSON(),this},toJSON:function(){return this._document}};var Q=["facebook","google","linkedIn","twitter"],R={use:C(Q)};Q.forEach(function(a){R[a]=B("Social."+a)}),e.Social={connect:function(a,b,c){var d;if(f.debug("Linking a social identity to a Kinvey user.",arguments),c=c||{},c.create="undefined"!=typeof c.create?c.create:!0,!e.Social.isSupported(b))return d=new e.Error("provider argument is not supported."),t(e.Defer.reject(d),c);var g=c.success;d=c.error,delete c.success,delete c.error;var h=R[b](c).then(function(f){a=a||{};var g=e.getActiveUser();if(a._socialIdentity=a._socialIdentity||{},a._socialIdentity[b]=f,null!==g){if(null==g._id)throw d=new e.Error("Active user does not have _id property defined.");if(g._id===a._id)return c._provider=b,e.User.update(a,c)}return a._socialIdentity={},a._socialIdentity[b]=f,e.User.login(a,null,c).then(null,function(b){return c.create&&e.Error.USER_NOT_FOUND===b.name?e.User.signup(a,c):e.Defer.reject(b)})});return h=h.then(function(a){return c.success=g,c.error=d,a}),h.then(function(a){f.debug("Linked the social identity to the Kinvey user.",a)},function(a){f.error("Failed to link a social identity to a Kinvey user.",a)}),t(h,c)},disconnect:function(a,b,c){var d;if(f.debug("Unlinking a social identity from a Kinvey user.",arguments),!e.Social.isSupported(b))return d=new e.Error("provider argument is not supported."),t(e.Defer.reject(d),c);if(a._socialIdentity=a._socialIdentity||{},a._socialIdentity[b]=null,null==a._id){var g=e.Defer.resolve(a);return t(g,c)}return e.User.update(a,c)},facebook:function(a,b){return f.debug("Linking a Facebook identity to a Kinvey user.",arguments),e.Social.connect(a,"facebook",b)},google:function(a,b){return f.debug("Linking a Google+ identity to a Kinvey user.",arguments),e.Social.connect(a,"google",b)},isSupported:function(a){return-1!==Q.indexOf(a)},linkedIn:function(a,b){return f.debug("Linking a LinkedIn identity to a Kinvey user.",arguments),e.Social.connect(a,"linkedIn",b)},twitter:function(a,b){return f.debug("Linking a Twitter identity to a Kinvey user.",arguments),e.Social.connect(a,"twitter",b)}},e.User={signup:function(a,b){return f.debug("Signing up a new user.",arguments),b=b||{},b.state=!0,e.User.create(a,b)},signupWithProvider:function(a,b,c){f.debug("Signing up a new user with a provider.",arguments);var d={_socialIdentity:{}};return d._socialIdentity[a]=b,e.User.signup(d,c)},login:function(a,b,c){var d;if(f.debug("Logging in an existing user.",arguments),x(a)?c="undefined"!=typeof c?c:b:a={username:a,password:b},c=c||{},(null==a.username||null==a.password)&&null==a._socialIdentity)return d=new e.Error("Username and/or password missing. Please provide both a username and password to login."),t(e.Defer.reject(d),c);if(null!==e.getActiveUser())return d=r(e.Error.ALREADY_LOGGED_IN),t(e.Defer.reject(d),c);var g=e.Persistence.create({namespace:k,collection:c._provider?null:"login",data:a,flags:c._provider?{provider:c._provider}:{},auth:H.App,local:{res:!0}},c).then(function(a){return e.setActiveUser(a),
a});return g.then(function(a){f.debug("Logged in the user.",a)},function(a){f.error("Failed to login the user.",a)}),t(g,c)},loginWithProvider:function(a,b,c){f.debug("Logging in with a provider.",arguments);var d={_socialIdentity:{}};return d._socialIdentity[a]=b,e.User.login(d,c)},logout:function(a){a=a||{};var b;return null===e.getActiveUser()?b=e.Defer.resolve(null):(f.debug("Logging out the active user.",arguments),b=e.Persistence.create({namespace:k,collection:"_logout",auth:H.Session},a).then(null,function(a){return e.Error.INVALID_CREDENTIALS===a.name||e.Error.EMAIL_VERIFICATION_REQUIRED===a.name?(f.error("The user credentials are invalid. Returning success because of the force flag."),null):e.Defer.reject(a)}).then(function(){return S.disconnect()}).then(function(){var a,b=e.setActiveUser(null);if(null==b._kmd)throw a=new e.Error("The previous active user does not have _kmd definedas a property.");return null!==b&&delete b._kmd.authtoken,b}),b.then(function(a){f.debug("Logged out the active user.",a)},function(a){f.error("Failed to logout the active user.",a)})),t(b,a)},me:function(a){f.debug("Retrieving information on the active user.",arguments),a=a||{};var b=e.Persistence.read({namespace:k,collection:"_me",auth:H.Session,local:{req:!0,res:!0}},a).then(function(a){return a._kmd=a._kmd||{},null==a._kmd.authtoken&&(a._kmd.authtoken=e.getActiveUser()._kmd.authtoken),e.setActiveUser(a),a});return b.then(function(a){f.debug("Retrieved information on the active user.",a)},function(a){f.error("Failed to retrieve information on the active user.",a)}),t(b,a)},verifyEmail:function(a,b){f.debug("Requesting e-mail verification.",arguments),b=b||{};var c=e.Persistence.create({namespace:j,collection:a,id:"user-email-verification-initiate",auth:H.App},b);return c.then(function(a){f.debug("Requested e-mail verification.",a)},function(a){f.error("Failed to request e-mail verification.",a)}),t(c,b)},forgotUsername:function(a,b){f.debug("Requesting a username reminder.",arguments),b=b||{};var c=e.Persistence.create({namespace:j,id:"user-forgot-username",data:{email:a},auth:H.App},b);return c.then(function(a){f.debug("Requested a username reminder.",a)},function(a){f.error("Failed to request a username reminder.",a)}),t(c,b)},resetPassword:function(a,b){f.debug("Requesting a password reset.",arguments),b=b||{};var c=e.Persistence.create({namespace:j,collection:a,id:"user-password-reset-initiate",auth:H.App},b);return c.then(function(a){f.debug("Requested a password reset.",a)},function(a){f.error("Failed to request a password reset.",a)}),t(c,b)},exists:function(a,b){f.debug("Checking whether a username exists.",arguments),b=b||{};var c=e.Persistence.create({namespace:j,id:"check-username-exists",data:{username:a},auth:H.App},b).then(function(a){return a.usernameExists});return c.then(function(a){f.debug("Checked whether the username exists.",a)},function(a){f.error("Failed to check whether the username exists.",a)}),t(c,b)},create:function(a,b){if(f.debug("Creating a new user.",arguments),b=b||{},!1!==b.state&&null!==e.getActiveUser()){var c=r(e.Error.ALREADY_LOGGED_IN);return t(e.Defer.reject(c),b)}var d=e.Persistence.create({namespace:k,data:a||{},auth:H.App},b).then(function(a){return!1!==b.state&&e.setActiveUser(a),a});return d.then(function(a){f.debug("Created the new user.",a)},function(a){f.error("Failed to create the new user.",a)}),t(d,b)},update:function(a,b){var c;if(f.debug("Updating a user.",arguments),null==a._id)return c=new e.Error("data argument must contain: _id"),t(e.Defer.reject(c),b);b=b||{};var d=[];if(null!=a._socialIdentity)for(var g in a._socialIdentity)a._socialIdentity.hasOwnProperty(g)&&null!=a._socialIdentity[g]&&g!==b._provider&&(d.push({provider:g,access_token:a._socialIdentity[g].access_token,access_token_secret:a._socialIdentity[g].access_token_secret}),delete a._socialIdentity[g].access_token,delete a._socialIdentity[g].access_token_secret);var h=e.Persistence.update({namespace:k,id:a._id,data:a,auth:H.Default,local:{res:!0}},b).then(function(a){d.forEach(function(b){var c=b.provider;null!=a._socialIdentity&&null!=a._socialIdentity[c]&&["access_token","access_token_secret"].forEach(function(d){null!=b[d]&&(a._socialIdentity[c][d]=b[d])})});var b=e.getActiveUser();if(null!==b){if(null==b._id)throw c=new e.Error("Active user does not have _id property defined.");if(null==a._id)throw c=new e.Error("User does not have _id property defined.");b._id===a._id&&(f.debug("Updating the active user because the updated user was the active user."),e.setActiveUser(a))}return a});return h.then(function(a){f.debug("Updated the user.",a)},function(a){f.error("Failed to update the user.",a)}),t(h,b)},find:function(a,b){var c;if(f.debug("Retrieving users by query.",arguments),null!=a&&!(a instanceof e.Query))return c=new e.Error("query argument must be of type: Kinvey.Query."),t(e.Defer.reject(c),b);b=b||{};var d;return b.discover?(f.debug("Using User Discovery because of the discover flag."),d=e.Persistence.create({namespace:k,collection:"_lookup",data:null!=a?a.toJSON().filter:null,auth:H.Default,local:{req:!0,res:!0}},b)):d=e.Persistence.read({namespace:k,query:a,auth:H.Default,local:{req:!0,res:!0}},b),d.then(function(a){f.debug("Retrieved the users by query.",a)},function(a){f.error("Failed to retrieve the users by query.",a)}),t(d,b)},get:function(a,b){f.debug("Retrieving a user.",arguments),b=b||{};var c=e.Persistence.read({namespace:k,id:a,auth:H.Default,local:{req:!0,res:!0}},b);return c.then(function(a){f.debug("Retrieved the user.",a)},function(a){f.error("Failed to return the user.",a)}),t(c,b)},destroy:function(a,b){var c;f.debug("Deleting a user.",arguments),b=b||{};var d=e.Persistence.destroy({namespace:k,id:a,flags:b.hard?{hard:!0}:{},auth:H.Default,local:{res:!0}},b).then(function(b){var d=e.getActiveUser();if(null!==d){if(null==d._id)throw c=new e.Error("Active user does not have _id property defined.");d._id===a&&(f.debug("Deleting the active user because the deleted user was the active user."),e.setActiveUser(null))}return b},function(a){return b.silent&&e.Error.USER_NOT_FOUND===a.name?(f.debug("The user does not exist. Returning success because of the silent flag."),null):e.Defer.reject(a)});return d.then(function(a){f.debug("Deleted the user.",a)},function(a){f.error("Failed to delete the user.",a)}),t(d,b)},restore:function(a,b){f.debug("Restoring a previously disabled user.",arguments),b=b||{};var c=e.Persistence.create({namespace:k,collection:a,id:"_restore",auth:H.Master},b);return c.then(function(a){f.debug("Restored the previously disabled user.",a)},function(a){f.error("Failed to restore the previously disabled user.",a)}),t(c,b)},count:function(a,b){var c;if(f.debug("Counting the number of users.",arguments),null!=a&&!(a instanceof e.Query))return c=new e.Error("query argument must be of type: Kinvey.Query."),t(e.Defer.reject(c),b);b=b||{};var d=e.Persistence.read({namespace:k,id:"_count",query:a,auth:H.Default,local:{req:!0}},b).then(function(a){return a.count});return d.then(function(a){f.debug("Counted the number of users.",a)},function(a){f.error("Failed to count the number of users.",a)}),t(d,b)},group:function(a,b){var c;if(f.debug("Grouping users.",arguments),!(a instanceof e.Group))return c=new e.Error("aggregation argument must be of type: Kinvey.Group."),t(e.Defer.reject(c),b);b=b||{};var d=e.Persistence.create({namespace:k,id:"_group",data:a.toJSON(),auth:H.Default,local:{req:!0}},b).then(function(b){return a.postProcess(b)});return d.then(function(a){f.debug("Grouped the users.",a)},function(a){f.error("Failed to group the users.",a)}),t(d,b)}};var S={AUTH_PATH:"/oauth/auth",TOKEN_PATH:"/oauth/token",AUTH_PROVIDER:"kinveyAuth",TOKEN_STORAGE_KEY:"micToken",AUTH_TIMEOUT:3e5,AuthorizationGrant:{AuthorizationCodeLoginPage:"AuthorizationCodeLoginPage",AuthorizationCodeAPI:"AuthorizationCodeAPI"},login:function(a,b,c){var d,f,g=e.appKey,h=e.getActiveUser();if(c=c||{},c.timeout=c.timeout||S.AUTH_TIMEOUT,c.attemptMICRefresh=!1,null!=h)return d=r(e.Error.ALREADY_LOGGED_IN),t(e.Defer.reject(d),c);if(null==b)return d=new e.Error("A redirect uri must be provided to login with MIC."),t(e.Defer.reject(d),c);if(S.AuthorizationGrant.AuthorizationCodeLoginPage===a){if(this.isNode())return d=new e.Error(S.AuthorizationGrant.AuthorizationCodeLoginPage+" grant is not supported."),t(e.Defer.reject(d),c);f=S.requestCodeWithPopup(g,b,c)}else{if(S.AuthorizationGrant.AuthorizationCodeAPI!==a)return d=r(e.Error.MIC_ERROR,{debug:"The authorization grant "+a+" is unrecognized. Please provide one of the following authorization grants: "+S.AuthorizationGrant.AuthorizationCodeLoginPage+", "+S.AuthorizationGrant.AuthorizationCodeAPI+"."}),t(e.Defer.reject(d),c);if(this.isHTML5()||this.isAngular()||this.isBackbone()||this.isPhoneGap()||this.isTitanium())return d=new e.Error(S.AuthorizationGrant.AuthorizationCodeAPI+" grant is not supported."),t(e.Defer.reject(d),c);if(null==c.username)return d=new e.Error("A username must be provided to login with MIC using the "+S.AuthorizationGrant.AuthorizationCodeAPI+" grant."),t(e.Defer.reject(d),c);if(null==c.password)return d=new e.Error("A password must be provided to login with MIC using the "+S.AuthorizationGrant.AuthorizationCodeAPI+" grant."),t(e.Defer.reject(d),c);f=S.requestUrl(g,b,c).then(function(a){return S.requestCodeWithUrl(a,g,b,c)})}return f=f.then(function(a){return S.requestToken(g,b,a,c)}).then(function(a){return c.create=!1===c.create?!1:!0,S.connect(h,a.access_token,c).then(function(c){return F.save(S.TOKEN_STORAGE_KEY,{refresh_token:a.refresh_token,redirect_uri:b}).then(function(){return c})})}),t(f,c)},refresh:function(a){var b,c,d,f=e.appKey,g=e.getActiveUser();return a=a||{},a.attemptMICRefresh=!1,d=F.get(S.TOKEN_STORAGE_KEY).then(function(d){if(null!=d)return c=d.redirect_uri,S.refreshToken(f,c,d.refresh_token,a);throw b=r(e.Error.MIC_ERROR,{debug:"Unable to refresh because there is not a valid refresh token available."})}).then(function(b){return S.connect(g,b.access_token,a).then(function(a){return F.save(S.TOKEN_STORAGE_KEY,{refresh_token:b.refresh_token,redirect_uri:c}).then(function(){return a})})},function(a){return F.destroy(S.TOKEN_STORAGE_KEY).then(function(){throw a})}),t(d,a)},requestUrl:function(a,b,c){var d=e.MICHostName;if(c.micApiVersion=c.micApiVersion||e.MICAPIVersion,null!=c.micApiVersion){var f=c.micApiVersion+"";f=0===f.indexOf("v")?f:"v"+f,d=d+"/"+f}var g={method:"POST",url:d+S.AUTH_PATH,data:{client_id:a,redirect_uri:b,response_type:"code"},headers:{"Content-Type":"application/x-www-form-urlencoded",Accept:"application/json","X-Kinvey-API-Version":e.API_VERSION,"X-Kinvey-Device-Information":I()}};return e.Log.getLevel()===e.Log.levels.TRACE&&(g.headers["X-Kinvey-Trace-Request"]="true",g.headers["X-Kinvey-Force-Debug-Log-Credentials"]="true"),e.Persistence.Net.request(g.method,g.url,S.encodeFormData(g.data),g.headers,c).then(function(a){try{a=JSON.parse(a)}catch(b){}return a.temp_login_uri})},requestCodeWithPopup:function(b,c,d){var f,g=e.Defer.deferred(),h=e.MICHostName;if(d.micApiVersion=d.micApiVersion||e.MICAPIVersion,null!=d.micApiVersion){var i=d.micApiVersion+"";i=0===i.indexOf("v")?i:"v"+i,h=h+"/"+i}h=h+S.AUTH_PATH+"?client_id="+encodeURIComponent(b)+"&redirect_uri="+encodeURIComponent(c)+"&response_type=code";var j,k,l,m=!1;d=d||{},d.url=d.url||h;var n=function(a){var b=!1;try{b=0===a.url.indexOf(c)}catch(d){}if(b){var e="?"+a.url.split("?")[1],f=E(e);g.resolve(f.code),m=!0,setTimeout(function(){j.close()},200)}},o=function(){m||(f=r(e.Error.MIC_ERROR,{debug:"The popup was closed without authorizing the user."}),g.reject(f)),S.isPhoneGap()?(j.removeEventListener("loadstart",n),j.removeEventListener("exit",o)):S.isTitanium()&&(k.removeEventListener("load",n),k.removeEventListener("error",n),j.removeEventListener("close",o),"iPhone OS"===Titanium.Platform.name?l.removeEventListener("click",p):"Android"===Titanium.Platform.name&&(j.close(),j.removeEventListener("androidback",o)))},p=function(){j.close()};if(S.isPhoneGap())j=a.open(d.url,"_blank","location=yes"),null==j?(f=r(e.Error.MIC_ERROR,{debug:"The popup was blocked."}),g.reject(f)):(j.addEventListener("loadstart",n),j.addEventListener("exit",o));else if(S.isTitanium()){if(k=Titanium.UI.createWebView({width:"100%",height:"100%",url:d.url}),j=Titanium.UI.createWindow({backgroundColor:"white",barColor:"#000",title:"Kinvey - MIC",modal:!0}),j.add(k),"iPhone OS"===Titanium.Platform.name){var q=Titanium.UI.createWindow({backgroundColor:"white",barColor:"#e3e3e3",title:"Kinvey - MIC"});q.add(k),l=Titanium.UI.createButton({title:"Close",style:Titanium.UI.iPhone.SystemButtonStyle.DONE}),q.setLeftNavButton(l),l.addEventListener("click",p),j=Titanium.UI.iOS.createNavigationWindow({backgroundColor:"white",window:q,modal:!0})}else"Android"===Titanium.Platform.name&&j.addEventListener("androidback",o);j.open(),k.addEventListener("load",n),k.addEventListener("error",n),j.addEventListener("close",o)}else{j=a.open(d.url,"_blank","toolbar=no,location=no");var s=0,t=100,u=a.setInterval(function(){if(null==j)a.clearTimeout(u),f=r(e.Error.MIC_ERROR,{debug:"The popup was blocked."}),g.reject(f);else if(j.closed)a.clearTimeout(u),f=r(e.Error.MIC_ERROR,{debug:"The popup was closed without authorizing the user."}),g.reject(f);else if(d.timeout&&s>d.timeout)a.clearTimeout(u),j.close(),f=r(e.Error.MIC_ERROR,{debug:"The authorization request timed out."}),g.reject(f);else{var b=!1;try{b=0===j.location.href.indexOf(c)}catch(h){}if(b){a.clearTimeout(u);var i=E(j.location.search);g.resolve(i.code),m=!0,j.close()}}s+=t},t)}return g.promise},requestCodeWithUrl:function(a,b,c,d){var f={method:"POST",url:a,data:{client_id:b,redirect_uri:c,response_type:"code",username:d.username,password:d.password},headers:{"Content-Type":"application/x-www-form-urlencoded",Accept:"application/json"}};return e.Persistence.Net.request(f.method,f.url,S.encodeFormData(f.data),f.headers,d).then(function(a){try{a=JSON.parse(a)}catch(b){}return a.code},function(a){throw a=r(e.Error.MIC_ERROR,{debug:"Unable to authorize user with username "+d.username+" and password "+d.password+"."})})},requestToken:function(a,b,c,d){var f={auth:H.App,method:"POST",url:e.MICHostName+S.TOKEN_PATH,data:{grant_type:"authorization_code",client_id:a,redirect_uri:b,code:c},headers:{"Content-Type":"application/x-www-form-urlencoded",Accept:"application/json","X-Kinvey-API-Version":e.API_VERSION,"X-Kinvey-Device-Information":I()}};return e.Log.getLevel()===e.Log.levels.TRACE&&(f.headers["X-Kinvey-Trace-Request"]="true",f.headers["X-Kinvey-Force-Debug-Log-Credentials"]="true"),f.auth().then(function(a){if(null!==a){var b=a.credentials;null!=a.username&&(b=e.Persistence.Net.base64(a.username+":"+a.password)),f.headers.Authorization=a.scheme+" "+b}return e.Persistence.Net.request(f.method,f.url,S.encodeFormData(f.data),f.headers,d)}).then(function(a){try{a=JSON.parse(a)}catch(b){}return a})},refreshToken:function(a,b,c,d){var f={auth:H.App,method:"POST",url:e.MICHostName+S.TOKEN_PATH,data:{grant_type:"refresh_token",client_id:a,redirect_uri:b,refresh_token:c},headers:{"Content-Type":"application/x-www-form-urlencoded",Accept:"application/json","X-Kinvey-API-Version":e.API_VERSION,"X-Kinvey-Device-Information":I()}};return e.Log.getLevel()===e.Log.levels.TRACE&&(f.headers["X-Kinvey-Trace-Request"]="true",f.headers["X-Kinvey-Force-Debug-Log-Credentials"]="true"),f.auth().then(function(a){if(null!==a){var b=a.credentials;null!=a.username&&(b=e.Persistence.Net.base64(a.username+":"+a.password)),f.headers.Authorization=a.scheme+" "+b}return e.Persistence.Net.request(f.method,f.url,S.encodeFormData(f.data),f.headers,d)}).then(function(a){try{a=JSON.parse(a)}catch(b){}return a})},connect:function(a,b,c){return a=a||{},e.setActiveUser(null),a._socialIdentity={},a._socialIdentity[S.AUTH_PROVIDER]={access_token:b},e.User.login(a,null,c).then(null,function(d){return c.create&&e.Error.USER_NOT_FOUND===d.name?e.User.signup(a,c).then(function(a){return S.connect(a,b,c)}):e.Defer.reject(d)})},disconnect:function(){return F.destroy(S.TOKEN_STORAGE_KEY)},encodeFormData:function(a){var b=[];for(var c in a)a.hasOwnProperty(c)&&b.push(encodeURIComponent(c)+"="+encodeURIComponent(a[c]));return b.join("&")},isHTML5:function(){return!(this.isTitanium()||this.isNode())},isAngular:function(){return"undefined"!=typeof a.angular},isBackbone:function(){return"undefined"!=typeof a.Backbone},isPhoneGap:function(){return"undefined"!=typeof a.cordova&&"undefined"!=typeof a.device},isTitanium:function(){return"undefined"!=typeof Titanium},isNode:function(){return"undefined"!=typeof process&&"undefined"!=typeof require}};e.User=e.User||{},e.User.MIC={loginWithAuthorizationCodeLoginPage:function(a,b){return S.login(S.AuthorizationGrant.AuthorizationCodeLoginPage,a,b)},loginWithAuthorizationCodeAPI:function(a,b,c,d){return d=d||{},d.username=a,d.password=b,S.login(S.AuthorizationGrant.AuthorizationCodeAPI,c,d)},logout:function(a){return e.User.logout(a)}},e.Query=function(a){a=a||{},this._fields=a.fields||[],this._filter=a.filter||{},this._sort=a.sort||{},this._limit=a.limit||null,this._skip=a.skip||0,this._parent=null},e.Query.prototype={equalTo:function(a,b){return this._filter[a]=b,this},contains:function(a,b){if(!u(b))throw new e.Error("values argument must be of type: Array.");return this._addFilter(a,"$in",b)},containsAll:function(a,b){if(!u(b))throw new e.Error("values argument must be of type: Array.");return this._addFilter(a,"$all",b)},greaterThan:function(a,b){if(!w(b)&&!z(b))throw new e.Error("value argument must be of type: number or string.");return this._addFilter(a,"$gt",b)},greaterThanOrEqualTo:function(a,b){if(!w(b)&&!z(b))throw new e.Error("value argument must be of type: number or string.");return this._addFilter(a,"$gte",b)},lessThan:function(a,b){if(!w(b)&&!z(b))throw new e.Error("value argument must be of type: number or string.");return this._addFilter(a,"$lt",b)},lessThanOrEqualTo:function(a,b){if(!w(b)&&!z(b))throw new e.Error("value argument must be of type: number or string.");return this._addFilter(a,"$lte",b)},notEqualTo:function(a,b){return this._addFilter(a,"$ne",b)},notContainedIn:function(a,b){if(!u(b))throw new e.Error("values argument must be of type: Array.");return this._addFilter(a,"$nin",b)},and:function(){return this._join("$and",Array.prototype.slice.call(arguments))},nor:function(){return null!==this._parent&&null!=this._parent._filter.$and?this._parent.nor.apply(this._parent,arguments):this._join("$nor",Array.prototype.slice.call(arguments))},or:function(){return null!==this._parent?this._parent.or.apply(this._parent,arguments):this._join("$or",Array.prototype.slice.call(arguments))},exists:function(a,b){return b="undefined"==typeof b?!0:b||!1,this._addFilter(a,"$exists",b)},mod:function(a,b,c){if(z(b)&&(b=parseFloat(b)),"undefined"==typeof c?c=0:z(c)&&(c=parseFloat(c)),!w(b))throw new e.Error("divisor arguments must be of type: number.");if(!w(c))throw new e.Error("remainder argument must be of type: number.");return this._addFilter(a,"$mod",[b,c])},matches:function(a,b,c){if(y(b)||(b=new RegExp(b)),c=c||{},(b.ignoreCase||c.ignoreCase)&&!1!==c.ignoreCase)throw new Error("ignoreCase flag is not supported.");if(0!==b.source.indexOf("^"))throw new Error("regExp must be an anchored expression.");var d=[];(b.multiline||c.multiline)&&!1!==c.multiline&&d.push("m"),c.extended&&d.push("x"),c.dotMatchesAll&&d.push("s");var e=this._addFilter(a,"$regex",b.source);return 0!==d.length&&this._addFilter(a,"$options",d.join("")),e},near:function(a,b,c){if(!u(b)||null==b[0]||null==b[1])throw new e.Error("coord argument must be of type: Array.<number, number>.");b[0]=parseFloat(b[0]),b[1]=parseFloat(b[1]);var d=this._addFilter(a,"$nearSphere",[b[0],b[1]]);return null!=c&&this._addFilter(a,"$maxDistance",c),d},withinBox:function(a,b,c){if(!u(b)||null==b[0]||null==b[1])throw new e.Error("bottomLeftCoord argument must be of type: Array.<number, number>.");if(!u(c)||null==c[0]||null==c[1])throw new e.Error("upperRightCoord argument must be of type: Array.<number, number>.");b[0]=parseFloat(b[0]),b[1]=parseFloat(b[1]),c[0]=parseFloat(c[0]),c[1]=parseFloat(c[1]);var d=[[b[0],b[1]],[c[0],c[1]]];return this._addFilter(a,"$within",{$box:d})},withinPolygon:function(a,b){if(!u(b)||3>b.length)throw new e.Error("coords argument must be of type: Array.Array.<number, number>.");return b=b.map(function(a){if(null==a[0]||null==a[1])throw new e.Error("coords argument must be of type: Array.Array.<number, number>.");return[parseFloat(a[0]),parseFloat(a[1])]}),this._addFilter(a,"$within",{$polygon:b})},size:function(a,b){if(z(b)&&(b=parseFloat(b)),!w(b))throw new e.Error("size argument must be of type: number.");return this._addFilter(a,"$size",b)},fields:function(a){if(a=a||[],!u(a))throw new e.Error("fields argument must be of type: Array.");return null!==this._parent?this._parent.fields(a):this._fields=a,this},limit:function(a){if(a=a||null,z(a)&&(a=parseFloat(a)),null!=a&&!w(a))throw new e.Error("limit argument must be of type: number.");return null!==this._parent?this._parent.limit(a):this._limit=a,this},skip:function(a){if(z(a)&&(a=parseFloat(a)),!w(a))throw new e.Error("skip argument must be of type: number.");return null!==this._parent?this._parent.skip(a):this._skip=a,this},ascending:function(a){return null!==this._parent?this._parent.ascending(a):this._sort[a]=1,this},descending:function(a){return null!==this._parent?this._parent.descending(a):this._sort[a]=-1,this},sort:function(a){if(null!=a&&!x(a))throw new e.Error("sort argument must be of type: Object.");return null!==this._parent?this._parent.sort(a):this._sort=a||{},this},toJSON:function(){return null!==this._parent?this._parent.toJSON():{fields:this._fields,filter:this._filter,sort:this._sort,skip:this._skip,limit:this._limit}},_addFilter:function(a,b,c){return x(this._filter[a])||(this._filter[a]={}),this._filter[a][b]=c,this},_join:function(a,b){var c=this;b=b.map(function(a){if(!(a instanceof e.Query)){if(!x(a))throw new e.Error("query argument must be of type: Kinvey.Query[] or Object[].");a=new e.Query(a)}return a.toJSON().filter}),0===b.length&&(c=new e.Query,b=[c.toJSON().filter],c._parent=this);var d={};for(var f in this._filter)this._filter.hasOwnProperty(f)&&(d[f]=this._filter[f],delete this._filter[f]);return this._filter[a]=[d].concat(b),c},_postProcess:function(a){if(!u(a))throw new e.Error("response argument must be of type: Array.");var b=this;return a=a.sort(function(a,c){for(var d in b._sort)if(b._sort.hasOwnProperty(d)){var e=s(a,d),f=s(c,d);if(null!=e&&null==f)return-1;if(null!=f&&null==e)return 1;if(e!==f){var g=b._sort[d];return(f>e?-1:1)*g}}return 0}),null!==this._limit?a.slice(this._skip,this._skip+this._limit):a.slice(this._skip)}};var T=function(a){var b={};for(var c in a)a.hasOwnProperty(c)&&(b[c]=a[c]);return b},U={get:function(a,b){if(f.debug("Retrieving relations for a document.",a,b),u(a)){var c=a.map(function(a){return U.get(a,T(b))});return e.Defer.all(c)}b=b||{},b.exclude=b.exclude||[],b.relations=b.relations||{};var d=b.error,g=b.relations,h=b.success;delete b.error,delete b.relations,delete b.success;var i={};Object.keys(g).forEach(function(a){var b=i,c=a.split("."),d=c.length;c.forEach(function(a,c){b.keys||(b.keys={}),b.keys[a]||(b.keys[a]={}),b=b.keys[a],c===d-1&&(b.resolve=!0)})});var j=function(a,c){var d;if(c.keys){var f=[];return Object.keys(c.keys).forEach(function(g){var h=c.keys[g];if(h.resolve){var i=u(a[g]),l=(i?a[g]:[a[g]]).map(function(a){if(null==a||"KinveyRef"!==a._type)return e.Defer.resolve(a);if(null==a._id)return d=new e.Error("Member does not have _id property defined. It is required to resolve the relationship."),e.Defer.reject(d);var c;return c=k===a._collection?e.User.get(a._id,b):e.DataStore.get(a._collection,a._id,b),c.then(function(a){return j(a,h).then(function(){return a},function(){return a})},function(){return e.Defer.resolve(a)})});f.push(e.Defer.all(l).then(function(b){a[g]=i?b:b[0]}))}else f.push(j(a[g],h))}),e.Defer.all(f)}return e.Defer.resolve()},l=j(a,i);return l.then(function(){return f.debug("Retrieved relations for the document.",a),b.error=d,b.relations=g,b.success=h,a},function(c){return f.error("Failed to retrieve relations for the document.",a),b.error=d,b.relations=g,b.success=h,e.Defer.reject(c)})},save:function(a,b,c){if(f.debug("Saving a document with relations.",a,b,c),u(b)){var d=b.map(function(b){return U.save(a,b,T(c))});return e.Defer.all(d)}c=c||{},c.exclude=c.exclude||[],c.relations=c.relations||{};var g=c.error,h=c.relations,i=c.success;delete c.error,delete c.relations,delete c.success;var j=[];h[""]=a,Object.keys(h).forEach(function(a){var b=""===a?0:a.split(".").length;j[b]=(j[b]||[]).concat(a)});var l={},m=e.Defer.resolve(null);return j.reverse().forEach(function(a){m=m.then(function(){var d=a.map(function(a){var d=h[a],f=s(b,a),g=u(f),i=(g?f:[f]).map(function(b){if(null==b||"KinveyRef"===b._type||-1!==c.exclude.indexOf(a))return e.Defer.resolve(b);var f,g=c.offline&&!1===c.track;if(k!==d||g)f=e.DataStore.save(d,b,c);else{var h=null==b._id;c.state=h&&""!==a?c.state||!1:c.state,f=e.User[h?"create":"update"](b,c)}return f.then(null,function(d){return c.force&&""!==a?b:e.Defer.reject(d)})});return e.Defer.all(i).then(function(c){var e=c.map(function(a){return null==a||null==a._id?a:{_type:"KinveyRef",_collection:d,_id:a._id}});g||(e=e[0],c=c[0]),s(b,a,e),l[a]=c})});return e.Defer.all(d)})}),m.then(function(){var a=l[""];return j.reverse().forEach(function(b){b.forEach(function(b){s(a,b,l[b])})}),f.debug("Saved the document with relations.",a),delete h[""],c.error=g,c.relations=h,c.success=i,a},function(a){return f.error("Failed to save the document with relations.",g),delete h[""],c.error=g,c.relations=h,c.success=i,e.Defer.reject(a)})}},V=function(a){var b=e.Sync.isEnabled(),c=e.Sync.isOnline();return a.fallback=b&&c&&!1!==a.fallback,a.offline=b&&(!c||a.offline),a.refresh=b&&c&&!1!==a.refresh,a.handler="function"==typeof a.handler?a.handler:function(){},a},W={addMetadata:function(a,b){var c=(new Date).toISOString(),d=u(a),e=d?a:[a];return e=e.map(function(a){return null!=a&&(a._kmd=a._kmd||{},a._kmd.lastRefreshedAt=c,null!=b&&(a._kmd.maxAge=b)),a}),d?e:e[0]},removeMetadata:function(a){var b=u(a),c=b?a:[a];return c=c.map(function(a){return null!=a&&null!=a._kmd&&(delete a._kmd.lastRefreshedAt,delete a._kmd.maxAge),a}),b?c:c[0]},status:function(a,b){for(var c=!1,d=u(a)?a:[a],f=d.length,g=(new Date).getTime(),h=0;f>h;h+=1){var i=d[h];if(null==i._kmd){var j=new e.Error("The item does not have _kmd defined as a property.It is required to get the maxAge status.");throw j}if(null!=i&&null!=i._kmd&&null!=i._kmd.lastRefreshedAt){var k=1e3*(b||i._kmd.maxAge),l=P(i._kmd.lastRefreshedAt).getTime(),m=l+k;if(g>m)return!1;var n=l+.9*k;g>n&&(c=!0)}}return c?{refresh:!0}:!0}};e.Persistence={create:function(a,b){if(b.relations){var c=k===a.namespace?k:a.collection;return U.save(c,a.data,b)}if(a.local=a.local||{},b=V(b),a.local.req&&b.offline)return f.debug("Using local persistence."),e.Persistence.Local.create(a,b).then(null,function(c){return b.fallback&&"_group"===a.id?(f.debug("Local persistence failed. Use net persistence because of the fallback flag."),b.offline=!1,e.Persistence.create(a,b)):e.Defer.reject(c)});f.debug("Using net persistence.");var d=e.Persistence.Net.create(a,b);return a.local.res&&b.refresh?(f.debug("Persisting the response locally."),d.then(function(c){return a.data=c,e.Persistence.Local.create(a,b).then(function(){return c})})):d},read:function(a,b){if(a.local=a.local||{},b=V(b),a.local.req&&b.offline)return f.debug("Using local persistence."),e.Persistence.Local.read(a,b).then(null,function(c){return b.fallback?(f.debug("Local persistence failed. Use net persistence because of the fallback flag."),b.offline=!1,e.Persistence.read(a,b)):e.Defer.reject(c)});f.debug("Using net persistence.");var c=e.Persistence.Net.read(a,b),d=b.fields||a.query&&!A(a.query._fields);return a.local.res&&b.refresh&&!d?c.then(function(c){f.debug("Persisting the response locally.");var d;if(b.relations){var g=b.offline;b.offline=!0,b.track=!1;var h=k===a.namespace?k:a.collection;d=U.save(h,c,b).then(function(){b.offline=g,delete b.track})}else a.data=c,d=e.Persistence.Local.create(a,b);return d.then(function(){return c})},function(c){return e.Error.ENTITY_NOT_FOUND===c.name?e.Persistence.Local.destroy(a,b).then(function(){return e.Defer.reject(c)}):e.Defer.reject(c)}):c},update:function(a,b){if(b.relations){var c=k===a.namespace?k:a.collection;return U.save(c,a.data,b)}if(a.local=a.local||{},b=V(b),a.local.req&&b.offline)return f.debug("Using local persistence."),e.Persistence.Local.update(a,b);f.debug("Using net persistence..");var d=e.Persistence.Net.update(a,b);return a.local.res&&b.refresh?(f.debug("Persisting the response locally."),d.then(function(c){return a.data=c,e.Persistence.Local.update(a,b).then(function(){return c})})):d},destroy:function(a,b){if(a.local=a.local||{},b=V(b),a.local.req&&b.offline)return f.debug("Using local persistence."),e.Persistence.Local.destroy(a,b);f.debug("Using net persistence.");var c=e.Persistence.Net.destroy(a,b);return a.local.res&&b.refresh?(f.debug("Persisting the response locally."),c.then(function(c){return e.Persistence.Local.destroy(a,b).then(function(){return c},function(a){return e.Error.ENTITY_NOT_FOUND===a.name?c:e.Defer.reject(a)})})):c}};var X={version:1,versionTable:"KinveyDatabaseVersion",upgrade:function(){try{return X.find(X.versionTable).then(null,function(){return[void 0]}).then(function(a){var b=a[0]||{};return X.onUpgrade(b.version,X.version).then(function(){return b})}).then(function(a){return a.version=X.version,X.save(X.versionTable,a)}).then(function(){})}catch(a){return e.Defer.resolve()}},onUpgrade:function(a,b){var c=e.Defer.deferred(),d=null==a?1:a+1;return b>=d?X.onUpgrade(d,b):(c.resolve(),c.promise)},batch:B("Database.batch"),clean:B("Database.clean"),count:B("Database.count"),destroy:B("Database.destroy"),destruct:B("Database.destruct"),find:B("Database.find"),findAndModify:B("Database.findAndModify"),get:B("Database.get"),group:B("Database.group"),save:B("Database.save"),update:B("Database.update"),isTemporaryObjectID:B("Database.isTemporaryObjectID"),use:C(["batch","clean","count","destroy","destruct","find","findAndModify","get","group","save","update","isTemporaryObjectID"])};e.Persistence.Local={create:function(a,b){f.debug("Initiating a create request.",arguments),b=b||{};var c=k===a.namespace?k:a.collection;if("_group"===a.id)return X.group(c,a.data,b);a.data=W.addMetadata(a.data,b.maxAge);var d=u(a.data)?"batch":"save",e=X[d](c,a.data,b);return e.then(function(a){return b.offline&&!1!==b.track?(f.debug("Notifying the synchronization functionality.",c,a),Z.notify(c,a,b).then(function(){return a})):a})},read:function(a,b){var c;f.debug("Initiating a read request.",arguments),b=b||{};var d=k===a.namespace?k:a.collection;if("_count"===a.id)return X.count(d,a.query,b);if("_me"===a.collection){var g=e.getActiveUser();return null!==g?null==g._id?(c=new e.Error("Active user does not have the _id property defined. Unable to retrieve information about the user."),e.Defer.reject(c)):X.get(d,g._id,b).then(null,function(a){return a.name===e.Error.ENTITY_NOT_FOUND?g:e.Defer.reject(a)}):(c=r(e.Error.NO_ACTIVE_USER),e.Defer.reject(c))}var h;return h=null==a.id?X.find(d,a.query,b):X.get(d,a.id,b),h.then(function(c){var d=W.status(c,b.maxAge);return!1===d&&e.Sync.isOnline()?(b.offline=!1,e.Persistence.read(a,b)):b.relations?U.get(c,b).then(function(c){return!0===d.refresh&&e.Sync.isOnline()&&(b.offline=!1,e.Persistence.read(a,b)),c}):(!0===d.refresh&&e.Sync.isOnline()&&(b.offline=!1,e.Persistence.read(a,b)),c)})},update:function(a,b){f.debug("Initiating an update request.",arguments),b=b||{};var c=k===a.namespace?k:a.collection;a.data=W.addMetadata(a.data,b.maxAge);var d=X.update(c,a.data,b);return d.then(function(a){return b.offline&&!1!==b.track?(f.debug("Notifying the synchronization functionality.",c,a),
Z.notify(c,a,b).then(function(){return a})):a})},destroy:function(a,b){f.debug("Initiating a delete request.",arguments),b=b||{};var c,d=k===a.namespace?k:a.collection;return c=null==a.id?X.clean(d,a.query,b):X.destroy(d,a.id,b),c.then(function(a){return b.offline&&!1!==b.track?(f.debug("Notifying the synchronization functionality.",d,a),Z.notify(d,a.documents,b).then(function(){return a})):a})}};var Y=null;e.Persistence.Net={create:function(a,b){return f.debug("Initiating a create request.",arguments),a.data=W.removeMetadata(a.data),a.method="POST",e.Persistence.Net._request(a,b)},read:function(a,b){if(f.debug("Initiating a read request.",arguments),a.flags=a.flags||{},b=b||{},u(b.fields)&&(a.flags.fields=b.fields.join(",")),null!=a.collection&&(!1!==b.fileTls&&(a.flags.kinveyfile_tls=!0),b.fileTtl&&(a.flags.kinveyfile_ttl=b.fileTtl)),b.relations){b.exclude=b.exclude||[];var c=Object.keys(b.relations).filter(function(a){return-1===b.exclude.indexOf(a)});0!==c.length&&(a.flags.retainReferences=!1,a.flags.resolve=c.join(","))}return a.method="GET",e.Persistence.Net._request(a,b)},update:function(a,b){return f.debug("Initiating an update request.",arguments),a.data=W.removeMetadata(a.data),a.method="PUT",e.Persistence.Net._request(a,b)},destroy:function(a,b){return f.debug("Initiating a delete request.",arguments),a.method="DELETE",e.Persistence.Net._request(a,b)},_request:function(a,b){var c;if(null==a.method)return c=new e.Error("request argument must contain: method."),t(e.Defer.reject(c),b);if(null==a.namespace)return c=new e.Error("request argument must contain: namespace."),t(e.Defer.reject(c),b);if(null==a.auth)return c=new e.Error("request argument must contain: auth."),t(e.Defer.reject(c),b);if(null==e.appKey&&H.None!==a.auth)return c=r(e.Error.MISSING_APP_CREDENTIALS),t(e.Defer.reject(c),b);if(null==e.masterSecret&&b.skipBL)return c=r(e.Error.MISSING_MASTER_CREDENTIALS),t(e.Defer.reject(c),b);b.trace=b.trace||e.Log.getLevel()===e.Log.levels.TRACE&&!1!==b.trace,b.attemptMICRefresh=!1===b.attemptMICRefresh?!1:!0;var d=[a.namespace,e.appKey,a.collection,a.id];d=d.filter(function(a){return null!=a}).map(e.Persistence.Net.encode);var g=[e.APIHostName].concat(d).join("/")+"/",h=a.flags||{};if(a.query){var i=a.query.toJSON();h.query=i.filter,A(i.fields)||(h.fields=i.fields.join(",")),null!==i.limit&&(h.limit=i.limit),0!==i.skip&&(h.skip=i.skip),A(i.sort)||(h.sort=i.sort)}!1!==b.nocache&&(h._=Math.random().toString(36).substr(2));var j=[];for(var k in h)if(h.hasOwnProperty(k)){var m=z(h[k])?h[k]:JSON.stringify(h[k]);j.push(e.Persistence.Net.encode(k)+"="+e.Persistence.Net.encode(m))}0<j.length&&(g+="?"+j.join("&")),null===Y&&(Y=I());var n={Accept:"application/json","X-Kinvey-API-Version":e.API_VERSION,"X-Kinvey-Device-Information":Y};b.clientAppVersion=b.clientAppVersion||e.ClientAppVersion.stringValue(),null!=b.clientAppVersion&&(n["X-Kinvey-Client-App-Version"]=b.clientAppVersion+""),null!=a.data&&(n["Content-Type"]="application/json; charset=utf-8"),b.contentType&&(n["X-Kinvey-Content-Type"]=b.contentType),b.skipBL&&(n["X-Kinvey-Skip-Business-Logic"]="true"),b.trace&&(n["X-Kinvey-Include-Headers-In-Response"]="X-Kinvey-Request-Id",n["X-Kinvey-ResponseWrapper"]="true"),b.customRequestProperties=b.customRequestProperties||{};var o=e.CustomRequestProperties.properties();if(null!=o&&Object.keys(o).forEach(function(a){b.customRequestProperties.hasOwnProperty(a)||(b.customRequestProperties[a]=o[a])}),Object.getOwnPropertyNames(b.customRequestProperties).length>0){var p=JSON.stringify(b.customRequestProperties),q=D(p);if(q>=l)return c=new e.Error("Custom request properties is "+q+" bytes. It must be less then "+l+" bytes."),t(e.Defer.reject(c),b);n["X-Kinvey-Custom-Request-Properties"]=p}e.Log.getLevel()===e.Log.levels.TRACE&&(n["X-Kinvey-Trace-Request"]="true");var s=a.auth().then(function(a){if(null!==a){var b=a.credentials;null!=a.username&&(b=e.Persistence.Net.base64(a.username+":"+a.password)),n.Authorization=a.scheme+" "+b}});return s.then(function(){b._originalRequest=a;var c=e.Persistence.Net.request(a.method,g,a.data,n,b).then(function(c){try{c=JSON.parse(c)}catch(d){}if(b.trace&&x(c)&&f.debug("Obtained the request ID.",c.headers["X-Kinvey-Request-Id"]),"GET"===a.method&&null!=a.collection&&"appdata"===a.namespace){var h,i=null!=a.id?!0:!1,j=b.trace&&x(c)?c.result:c;if(u(j)&&i)throw h=new e.Error("Expected a single entity as a response to "+a.method+" "+g+". Received an array of entities instead.");if(!u(j)&&!i)throw h=new e.Error("Expected an array of entities as a response to "+a.method+" "+g+". Received a single entity instead.")}return b.trace&&x(c)?c.result:c},function(a){try{a=JSON.parse(a)}catch(c){}var d=null;if(b.trace&&(d=a.headers["X-Kinvey-Request-Id"],a=a.result),null!=a&&null!=a.error)a={name:a.error,description:a.description||"",debug:a.debug||""},b.trace&&(a.requestId=d,f.debug("Obtained the request ID.",d));else{var g={abort:e.Error.REQUEST_ABORT_ERROR,error:e.Error.REQUEST_ERROR,timeout:e.Error.REQUEST_TIMEOUT_ERROR};a=r(g[a]||g.error,{debug:a})}return e.Defer.reject(a)});return c.then(null,function(a){if(e.Error.USER_LOCKED_DOWN===a.name){if(e.setActiveUser(null),"undefined"!=typeof X){var b=function(){e.Defer.reject(a)};return e.Sync.destruct().then(b,b)}}else if(e.Error.INVALID_CREDENTIALS===a.name){var c=e.getActiveUser();a.debug+=null!=c&&null!=c._socialIdentity&&null!=c._socialIdentity[S.AUTH_PROVIDER]?" It is possible the tokens used to execute the request are expired. In that case, please execute `Kinvey.User.logout({ force: true })`, and then log back in using `Kinvey.User.MIC.loginWithAuthorizationCodeLoginPage(redirectUri)` or `Kinvey.User.MIC.loginWithAuthorizationCodeAPI(username, password, redirectUri)` to solve this issue.":" It is possible the tokens used to execute the request are expired. In that case, please execute `Kinvey.User.logout({ force: true })`, and then log back in using `Kinvey.User.login(username, password)` to solve this issue."}return e.Defer.reject(a)})})},base64:B("Kinvey.Persistence.Net.base64"),encode:B("Kinvey.Persistence.Net.encode"),request:B("Kinvey.Persistence.Net.request"),use:C(["base64","encode","request"])};var Z={enabled:!1,online:!0,system:"system.sync",count:function(a,b){if(b=b||{},null!=a)return X.get(Z.system,a,b).then(function(a){return a.size},function(a){return e.Error.ENTITY_NOT_FOUND===a.name?0:e.Defer.reject(a)});var c=e.Group.sum("size").toJSON();return X.group(Z.system,c,b).then(function(a){return a[0]?a[0].result:0})},execute:function(a){var b=(new e.Query).greaterThan("size",0);return X.find(Z.system,b,a).then(function(b){var c=b.map(function(b){return Z._collection(b._id,b.documents,a)});return e.Defer.all(c)})},notify:function(a,b,c){var d;return X.findAndModify(Z.system,a,function(f){return b=u(b)?b:[b],f=f||{_id:a,documents:{},size:0},b.forEach(function(a){if(null==a._id)throw d=new e.Error("Document does not have _id property defined. It is required to do proper synchronization.");f.documents.hasOwnProperty(a._id)||(f.size+=1);var b=null!=a._kmd?a._kmd.lmt:null,g=c.clientAppVersion||e.ClientAppVersion.stringValue(),h=c.customRequestProperties||{},i=e.CustomRequestProperties.properties();null!=i&&Object.keys(i).forEach(function(a){h.hasOwnProperty(a)||(h[a]=i[a])}),f.documents[a._id]={timestamp:b,clientAppVersion:g,customRequestProperties:h}}),f},c).then(function(){return null})},_collection:function(a,b,c){var d={collection:a,success:[],error:[]},f=Object.keys(b);return Z._read(a,b,c).then(function(g){var h=f.map(function(e){var f=b[e]||{},h={id:e,timestamp:f.timestamp,clientAppVersion:f.clientAppVersion,customRequestProperties:f.customRequestProperties};return Z._document(a,h,g.local[e]||null,g.net[e]||null,c).then(null,function(a){return d.error.push({id:a.id,error:a}),null})});return e.Defer.all(h)}).then(function(b){var d=b.filter(function(a){return null!=a&&null!==a.document}),f=b.filter(function(a){return null!=a&&null===a.document}),g=[Z._save(a,d,c),Z._destroy(a,f,c)];return e.Defer.all(g)}).then(function(b){return d.success=d.success.concat(b[0].success,b[1].success),d.error=d.error.concat(b[0].error,b[1].error),X.findAndModify(Z.system,a,function(a){d.success.forEach(function(b){a.documents.hasOwnProperty(b)&&(a.size-=1,delete a.documents[b])});try{var b=Object.keys(a.documents);b.forEach(function(b){a.documents.hasOwnProperty(b)&&-1===d.error.indexOf(b)&&(a.size-=1,delete a.documents[b])})}catch(c){}return a},c)}).then(function(){return d})},_read:function(a,b,c){var d=Object.keys(b),f=d.map(function(d){var f=b[d],g=c||{};g.clientAppVersion=null!=f&&null!=f.clientAppVersion?f.clientAppVersion:null,g.customRequestProperties=null!=f&&null!=f.customRequestProperties?f.customRequestProperties:null;var i={namespace:k===a?k:h,collection:k===a?null:a,query:(new e.Query).contains("_id",[d]),auth:H.Default};return e.Defer.all([e.Persistence.Local.read(i,g),e.Persistence.Net.read(i,g)]).then(function(a){return{local:a[0],net:a[1]}})});return e.Defer.all(f).then(function(a){var b,c={local:{},net:{}};return a.forEach(function(a){var d=a.local,f=a.net;d.forEach(function(a){if(null==a._id)throw b=new e.Error("Document does not have _id property defined. It is required to do proper synchronization.");c.local[a._id]=a}),f.forEach(function(a){if(null==a._id)throw b=new e.Error("Document does not have _id property defined. It is required to do proper synchronization.");c.net[a._id]=a})}),c})},_destroy:function(a,b,c){var d=b.map(function(b){var d=b.id,f=b.metadata,g=c||{};g.clientAppVersion=null!=f.clientAppVersion?f.clientAppVersion:null,g.customRequestProperties=null!=f.customRequestProperties?f.customRequestProperties:null;var i={namespace:k===a?k:h,collection:k===a?null:a,query:(new e.Query).contains("_id",[d]),auth:H.Default},j=[e.Persistence.Local.destroy(i,g),e.Persistence.Net.destroy(i,g)];return e.Defer.all(j).then(function(){return{success:[d],error:[]}},function(a){return{success:[],error:[{id:d,error:a}]}})});return e.Defer.all(d).then(function(a){var b={success:[],error:[]};return a.forEach(function(a){b.success=b.success.concat(a.success),b.error=b.error.concat(a.error)}),b})},_document:function(a,b,c,d,f){var g;return null==b?(g=new e.Error("Missing required metadata for the document in collection "+a+". This is required to properly sync."),e.Defer.reject(g)):null==b.id?(g=new e.Error("Metadata does not have id defined. This is required to properly sync the document in collection "+a+"."),e.Defer.reject(g)):null===d||null!=d._kmd&&b.timestamp===d._kmd.lmt?e.Defer.resolve({id:b.id,document:c,metadata:b}):null!=f.conflict?f.conflict(a,c,d).then(function(a){return{id:b.id,document:a,metadata:b}},function(){return e.Defer.reject({id:b.id,document:[c,d],metadata:b})}):e.Defer.reject({id:b.id,document:[c,d],metadata:b})},_save:function(a,b,c){var d=[],f=b.map(function(b){var f=b.document,g=b.metadata,i=c||{},j=f._id;return i.clientAppVersion=null!=g.clientAppVersion?g.clientAppVersion:null,i.customRequestProperties=null!=g.customRequestProperties?g.customRequestProperties:null,X.isTemporaryObjectID(j)?(delete f._id,e.Persistence.Net.create({namespace:k===a?k:h,collection:k===a?null:a,data:f,auth:H.Default},i).then(function(b){return X.destroy(a,j).then(function(){return b})},function(a){return f._id=j,d.push({id:f._id,error:a}),null})):e.Persistence.Net.update({namespace:k===a?k:h,collection:k===a?null:a,id:f._id,data:f,auth:H.Default},i).then(null,function(a){return d.push({id:f._id,error:a}),null})});return e.Defer.all(f).then(function(b){return e.Persistence.Local.create({namespace:k===a?k:h,collection:k===a?null:a,data:b,auth:H.Default},c)}).then(function(a){return{success:a.map(function(a){return a._id}),error:d}},function(a){return{success:[],error:b.map(function(b){return{id:b._id,error:a}})}})}};e.Sync={count:function(a,b){f.debug("Counting the number of documents pending synchronization.",arguments),b=b||{};var c=Z.count(a,b);return c.then(function(a){f.debug("Counted the number of documents pending synchronization.",a)},function(a){f.error("Failed to count the number of documents pending synchronization.",a)}),t(c,b)},destruct:function(a){f.debug("Deleting the local database.",arguments),a=a||{};var b=X.destruct(a);return b.then(function(a){f.debug("Deleted the local database.",a)},function(a){f.error("Failed to delete the local database.",a)}),t(b,a)},execute:function(a){if(f.debug("Synchronizing the application.",arguments),!e.Sync.isOnline()){var b=r(e.Error.SYNC_ERROR,{debug:"Sync is not enabled, or the application resides in offline mode."});return e.Defer.reject(b)}a=a||{};var c;return null!=a.user?(f.debug("Attempting to login with a user context.",a.user),c=e.User.login(a.user).then(function(){return delete a.user,e.Sync.execute(a)}),c.then(null,function(a){f.error("Failed to login with the user context.",a)}),delete a.success,t(c,a)):(c=Z.execute(a),c.then(function(a){f.debug("Synchonized the application.",a)},function(a){f.error("Failed to synchronize the application.",a)}),t(c,a))},init:function(a){return f.debug("Initializing the synchronization functionality.",arguments),a=a||{},Z.enabled="undefined"!=typeof a.enable?a.enable:Z.enabled,Z.online="undefined"!=typeof a.online?a.online:Z.online,e.Defer.resolve(null)},isEnabled:function(){return Z.enabled},isOnline:function(){return Z.online},offline:function(){if(f.debug("Switching the application state to offline."),!e.Sync.isEnabled()){var a=r(e.Error.SYNC_ERROR,{debug:"Sync is not enabled."});return e.Defer.reject(a)}return Z.online=!1,e.Defer.resolve(null)},online:function(a){if(f.debug("Switching the application state to online.",arguments),!e.Sync.isEnabled()){var b=r(e.Error.SYNC_ERROR,{debug:"Sync is not enabled."});return e.Defer.reject(b)}a=a||{};var c=Z.online;return Z.online=!0,!1!==a.sync&&c!==Z.online?e.Sync.execute(a):e.Defer.resolve(null)},clientAlwaysWins:function(a,b){return e.Defer.resolve(b)},serverAlwaysWins:function(a,b,c){return e.Defer.resolve(c)}},"undefined"!=typeof a.Promise&&e.Defer.use({deferred:function(){var b={};return b.promise=new a.Promise(function(a,c){b.resolve=a,b.reject=c}),b}}),a.Queue.configure(function(a){var b=e.Defer.deferred();try{a(b.resolve,b.reject,b.progress)}catch(c){b.reject(c)}return b.promise});var $={db:null,dbName:function(){if(null==e.appKey)throw new e.Error("Kinvey.appKey must not be null.");return"Kinvey."+e.appKey},objectIdPrefix:"temp_",size:5e6,queue:new a.Queue(1,1/0),open:function(){return a.openDatabase($.dbName(),1,"",$.size)},transaction:function(a,b,c,d){var g;if(!z(a)||!/^[a-zA-Z0-9\-]{1,128}/.test(a))return g=r(e.Error.INVALID_IDENTIFIER,{description:"The collection name has an invalid format.",debug:'The collection name must be a string containing only alphanumeric characters and dashes, "'+a+'" given.'}),e.Defer.reject(g);var h='"'+a+'"',i="sqlite_master"===a,j=u(b);b=j?b:[[b,c]],d=d||!1,null===$.db&&($.db=$.open());var k=e.Defer.deferred(),l=d||!v($.db.readTransaction);return $.db[l?"transaction":"readTransaction"](function(a){d&&!i&&a.executeSql("CREATE TABLE IF NOT EXISTS "+h+" (key BLOB PRIMARY KEY NOT NULL, value BLOB NOT NULL)");var c=b.length,e=[];b.forEach(function(b){var d=b[0].replace("#{collection}",h);f.debug("Executing a query.",d,b[1]),a.executeSql(d,b[1],function(a,g){var h={rowCount:g.rowsAffected,result:[]};if(g.rows.length)for(var l=0;l<g.rows.length;l+=1){var m=g.rows.item(l).value,n=i?m:JSON.parse(m);h.result.push(n)}e.push(h),f.debug("Executed the query.",d,b[1],h),c-=1,0===c&&k.resolve(j?e:e.shift())})})},function(b){if(f.error("Failed to execute the query.",b),b=z(b)?b:b.message,b&&-1!==b.indexOf("no such table"))return g=r(e.Error.COLLECTION_NOT_FOUND,{description:"This collection not found for this app backend",debug:{collection:a}}),k.reject(g);var c="SELECT name AS value FROM #{collection} WHERE type = ? AND name = ?",d=["table",a];return $.transaction("sqlite_master",c,d,!1).then(function(c){return g=0===c.result.length?r(e.Error.COLLECTION_NOT_FOUND,{description:"This collection not found for this app backend",debug:{collection:a}}):r(e.Error.DATABASE_ERROR,{debug:b}),k.reject(g)},function(){return g=r(e.Error.DATABASE_ERROR,{debug:b}),k.reject(g)})}),k.promise},objectID:function(a){a=a||24;for(var b="abcdef0123456789",c="",d=0,e=b.length;a>d;d+=1){var f=Math.floor(Math.random()*e);c+=b.substring(f,f+1)}return c=$.objectIdPrefix+c},isTemporaryObjectID:function(a){return null!=a?0===a.indexOf($.objectIdPrefix):!1},batch:function(a,b,c){if(0===b.length)return e.Defer.resolve(b);var d=[];b=b.map(function(a){return a._id=a._id||$.objectID(),d.push(["REPLACE INTO #{collection} (key, value) VALUES (?, ?)",[a._id,JSON.stringify(a)]]),a});var f=$.transaction(a,d,null,!0,c);return f.then(function(){return b})},clean:function(a,b,c){return $.queue.add(function(){var d;return null!=b&&b.sort(null).limit(null).skip(0),$.find(a,b,c).then(function(b){if(0===b.length)return{count:0,documents:[]};var f=[],g=b.map(function(a){if(null==a._id)throw d=new e.Error("Document does not have _id property defined. Unable to clean database.");return f.push("?"),a._id}),h="DELETE FROM #{collection} WHERE key IN("+f.join(",")+")",i=$.transaction(a,h,g,!0,c);return i.then(function(a){return a.rowCount=null!=a.rowCount?a.rowCount:b.length,{count:a.rowCount,documents:b}})})})},count:function(a,b,c){return null!=b&&b.sort(null).limit(null).skip(0),$.find(a,b,c).then(function(a){return{count:a.length}})},destroy:function(a,b,c){var d=$.transaction(a,[["SELECT value FROM #{collection} WHERE key = ?",[b]],["DELETE       FROM #{collection} WHERE key = ?",[b]]],null,!0,c);return d.then(function(c){var d=c[1].rowCount,f=c[0].result;if(d=null!=d?d:c[0].result.length,0===d){var g=r(e.Error.ENTITY_NOT_FOUND,{description:"This entity not found in the collection",debug:{collection:a,id:b}});return e.Defer.reject(g)}return{count:d,documents:f}})},destruct:function(a){return $.queue.add(function(){var b="SELECT name AS value FROM #{collection} WHERE type = ?",c=["table"],d=$.transaction("sqlite_master",b,c,!1,a);return d.then(function(b){var c=b.result;if(0===c.length)return null;var d=c.filter(function(a){return/^[a-zA-Z0-9\-]{1,128}/.test(a)}).map(function(a){return["DROP TABLE IF EXISTS '"+a+"'"]});return $.transaction("sqlite_master",d,null,!0,a)}).then(function(){return null})})},find:function(b,c,d){var f="SELECT value FROM #{collection}",g=$.transaction(b,f,[],!1,d);return g.then(function(b){return b=b.result,null==c?b:(b=a.sift(c.toJSON().filter,b),c._postProcess(b))},function(a){return e.Error.COLLECTION_NOT_FOUND===a.name?[]:e.Defer.reject(a)})},findAndModify:function(a,b,c,d){return $.queue.add(function(){var f=$.get(a,b,d).then(null,function(a){return e.Error.ENTITY_NOT_FOUND===a.name?null:e.Defer.reject(a)});return f.then(function(b){var e=c(b);return $.save(a,e,d)})})},get:function(a,b,c){var d="SELECT value FROM #{collection} WHERE key = ?",f=$.transaction(a,d,[b],!1,c);return f.then(function(c){var d=c.result;if(0===d.length){var f=r(e.Error.ENTITY_NOT_FOUND,{description:"This entity not found in the collection",debug:{collection:a,id:b}});return e.Defer.reject(f)}return d[0]},function(c){return e.Error.COLLECTION_NOT_FOUND===c.name&&(c=r(e.Error.ENTITY_NOT_FOUND,{description:"This entity not found in the collection",debug:{collection:a,id:b}})),e.Defer.reject(c)})},group:function(a,b,c){var d=b.reduce.replace(/function[\s\S]*?\([\s\S]*?\)/,"");b.reduce=new Function(["doc","out"],d);var f=new e.Query({filter:b.condition});return $.find(a,f,c).then(function(a){var c={};a.forEach(function(a){var d={};for(var e in b.key)b.key.hasOwnProperty(e)&&(d[e]=a[e]);var f=JSON.stringify(d);if(null==c[f]){c[f]=d;for(var g in b.initial)b.initial.hasOwnProperty(g)&&(c[f][g]=b.initial[g])}b.reduce(a,c[f])});var d=[];for(var e in c)c.hasOwnProperty(e)&&d.push(c[e]);return d})},save:function(a,b,c){b._id=b._id||$.objectID();var d="REPLACE INTO #{collection} (key, value) VALUES (?, ?)",e=[b._id,JSON.stringify(b)],f=$.transaction(a,d,e,!0,c);return f.then(function(){return b})},update:function(a,b,c){return $.save(a,b,c)}};"undefined"!=typeof a.cordova?document.addEventListener("deviceready",c,!1):c();var _={db:null,dbName:function(){if(null==e.appKey)throw new e.Error("Kinvey.appKey must not be null.");return"Kinvey."+e.appKey},objectIdPrefix:"temp_",impl:a.indexedDB||a.webkitIndexedDB||a.mozIndexedDB||a.oIndexedDB||a.msIndexedDB,inTransaction:!1,objectID:function(a){a=a||24;for(var b="abcdef0123456789",c="",d=0,e=b.length;a>d;d+=1){var f=Math.floor(Math.random()*e);c+=b.substring(f,f+1)}return c=_.objectIdPrefix+c},isTemporaryObjectID:function(a){return null!=a?0===a.indexOf(_.objectIdPrefix):!1},pending:[],transaction:function(a,b,c,d,f){if(!z(a)||!/^[a-zA-Z0-9\-]{1,128}/.test(a))return d(r(e.Error.INVALID_IDENTIFIER,{description:"The collection name has an invalid format.",debug:'The collection name must be a string containing only alphanumeric characters and dashes, "'+a+'" given.'}));if(b=b||!1,null!==_.db&&(!0===f||!_.inTransaction)){if(_.db.objectStoreNames.contains(a)){var g=b?"readwrite":"readonly",h=_.openTransactionSafely(_.db,[a],g);if(h.error&&d(h.error),null!=h.txn){var i=h.txn,j=i.objectStore(a);return c(j)}return d(new e.Error("Unable to open a transaction for the database. Please try this database transaction again."))}if(!b)return d(r(e.Error.COLLECTION_NOT_FOUND,{description:"This collection not found for this app backend",debug:{collection:a}}))}if(!0!==f&&_.inTransaction)return _.pending.push(function(){_.transaction(a,b,c,d)});_.inTransaction=!0;var k;if(null!==_.db){var l=_.db.version+1;_.db.close(),k=_.impl.open(_.dbName(),l)}else{if(null==e.appKey)return _.inTransaction=!1,d(r(e.Error.MISSING_APP_CREDENTIALS));k=_.impl.open(_.dbName())}k.onupgradeneeded=function(){_.db=k.result,b&&_.db.createObjectStore(a,{keyPath:"_id"})},k.onsuccess=function(){_.db=k.result,_.db.onversionchange=function(){null!==_.db&&(_.db.close(),_.db=null)};var e=function(a){return function(b){var c=a(b);if(_.inTransaction=!1,0!==_.pending.length){var d=_.pending;_.pending=[],d.forEach(function(a){a()})}return c}};_.transaction(a,b,e(c),e(d),!0)},k.onerror=function(a){d(r(e.Error.DATABASE_ERROR,{debug:a}))}},openTransactionSafely:function(a,b,c){try{return{txn:a.transaction(b,c)}}catch(d){return{error:d}}},batch:function(a,b){if(0===b.length)return e.Defer.resolve(b);var c=e.Defer.deferred();return _.transaction(a,!0,function(a){var d=a.transaction;b.forEach(function(b){b._id=b._id||_.objectID(),a.put(b)}),d.oncomplete=function(){c.resolve(b)},d.onerror=function(a){var b=r(e.Error.DATABASE_ERROR,{debug:a});c.reject(b)}},function(a){c.reject(a)}),c.promise},clean:function(a,b,c){var d;return null!=b&&b.sort(null).limit(null).skip(0),_.find(a,b,c).then(function(b){if(0===b.length)return{count:0,documents:[]};var c=e.Defer.deferred();return _.transaction(a,!0,function(a){var f=a.transaction;b.forEach(function(b){if(null==b._id)throw d=new e.Error("Document does not have _id property defined. Unable to clean database.");a["delete"](b._id)}),f.oncomplete=function(){c.resolve({count:b.length,documents:b})},f.onerror=function(a){var b=r(e.Error.DATABASE_ERROR,{debug:a});c.reject(b)}}),c.promise})},count:function(a,b,c){return null!=b&&b.sort(null).limit(null).skip(0),_.find(a,b,c).then(function(a){return{count:a.length}})},destroy:function(a,b){var c=e.Defer.deferred();return _.transaction(a,!0,function(d){var f=d.transaction,g=d.get(b);d["delete"](b),f.oncomplete=function(){return null==g.result?c.reject(r(e.Error.ENTITY_NOT_FOUND,{description:"This entity not found in the collection",debug:{collection:a,id:b}})):void c.resolve({count:1,documents:[g.result]})},f.onerror=function(a){var b=r(e.Error.DATABASE_ERROR,{debug:a});c.reject(b)}},function(a){c.reject(a)}),c.promise},destruct:function(){if(null==e.appKey){var a=r(e.Error.MISSING_APP_CREDENTIALS);return e.Defer.reject(a)}var b=e.Defer.deferred();null!==_.db&&(_.db.close(),_.db=null);var c=_.impl.deleteDatabase(_.dbName());return c.onsuccess=function(){b.resolve(null)},c.onerror=function(a){var c=r(e.Error.DATABASE_ERROR,{debug:a});b.reject(c)},b.promise},find:function(b,c){var d=e.Defer.deferred();return _.transaction(b,!1,function(a){var b=a.openCursor(),c=[];b.onsuccess=function(){var a=b.result;null!=a?(c.push(a.value),a["continue"]()):d.resolve(c)},b.onerror=function(a){d.reject(r(e.DATABASE_ERROR,{debug:a}))}},function(a){return e.Error.COLLECTION_NOT_FOUND===a.name?d.resolve([]):d.reject(a)}),d.promise.then(function(b){return null==c?b:(b=a.sift(c.toJSON().filter,b),c._postProcess(b))})},findAndModify:function(a,b,c){var d=e.Defer.deferred();return _.transaction(a,!0,function(a){var f=null,g=a.get(b);g.onsuccess=function(){f=c(g.result||null),a.put(f)};var h=a.transaction;h.oncomplete=function(){d.resolve(f)},h.onerror=function(a){var b=r(e.Error.DATABASE_ERROR,{debug:a});d.reject(b)}},function(a){d.reject(a)}),d.promise},get:function(a,b){var c=e.Defer.deferred();return _.transaction(a,!1,function(d){var f=d.get(b);f.onsuccess=function(){return null!=f.result?c.resolve(f.result):void c.reject(r(e.Error.ENTITY_NOT_FOUND,{description:"This entity not found in the collection",debug:{collection:a,id:b}}))},f.onerror=function(a){c.reject(r(e.Error.DATABASE_ERROR,{debug:a}))}},function(d){e.Error.COLLECTION_NOT_FOUND===d.name&&(d=r(e.Error.ENTITY_NOT_FOUND,{description:"This entity not found in the collection",debug:{collection:a,id:b}})),c.reject(d)}),c.promise},group:function(a,b,c){var d=b.reduce.replace(/function[\s\S]*?\([\s\S]*?\)/,"");b.reduce=new Function(["doc","out"],d);var f=new e.Query({filter:b.condition});return _.find(a,f,c).then(function(a){var c={};a.forEach(function(a){var d={};for(var e in b.key)b.key.hasOwnProperty(e)&&(d[e]=a[e]);var f=JSON.stringify(d);if(null==c[f]){c[f]=d;for(var g in b.initial)b.initial.hasOwnProperty(g)&&(c[f][g]=b.initial[g])}b.reduce(a,c[f])});var d=[];for(var e in c)c.hasOwnProperty(e)&&d.push(c[e]);return d})},save:function(a,b){b._id=b._id||_.objectID();var c=e.Defer.deferred();return _.transaction(a,!0,function(a){var d=a.put(b);d.onsuccess=function(){c.resolve(b)},d.onerror=function(a){var b=r(e.Error.DATABASE_ERROR,{debug:a});c.reject(b)}},function(a){c.reject(a)}),c.promise},update:function(a,b,c){return _.save(a,b,c)}};d();var aa={responseType:function(){try{return new a.Blob&&"blob"}catch(b){return"arraybuffer"}}(),supportsTimeout:XMLHttpRequest.prototype.hasOwnProperty("timeout"),base64:function(b){return a.btoa(b)},encode:a.encodeURIComponent,request:function(b,c,d,g,h){d=d||null,g=g||{},h=h||{};var i=e.Defer.deferred(),j=new XMLHttpRequest;j.open(b,c),h.file&&(j.responseType=aa.responseType,j.setRequestHeader("Accept",h.file)),0<h.timeout&&(j.timeout=h.timeout);var k=null,l=!1;if(0===c.indexOf(e.APIHostName)&&"GET"===b){var m=a.location;null!=m&&null!=m.protocol&&(g["X-Kinvey-Origin"]=m.protocol+"//"+m.host)}for(var n in g)g.hasOwnProperty(n)&&j.setRequestHeader(n,g[n]);j.onabort=j.onerror=j.onload=j.ontimeout=function(b){null!==k&&a.clearTimeout(k),f.debug("The network request completed.",j);var c=j.response||null;"undefined"==typeof j.response&&(c=j.responseText||null);var d=j.status;if(!h.file&&null!=c&&2===parseInt(d/100,10)&&204!==d){var g,m=j.getResponseHeader("Content-Type");if(null==m?g=new e.Error("Content-Type header missing in response. Please add Content-Type header to response with value application/json."):-1===m.indexOf("application/json")&&(g=new e.Error("Response Content-Type header is set to "+m+". Expected it to be set to application/json.")),g)return i.reject(g)}if(2===parseInt(d/100,10)||304===d)i.resolve(c);else{var n,o=h._originalRequest;n=401===d&&h.attemptMICRefresh?S.refresh(h):e.Defer.reject(),n.then(function(){return h.attemptMICRefresh=!1,e.Persistence.Net._request(o,h)}).then(function(a){i.resolve(a)},function(){var f=null!==k?"timeout":l===!0?"canceled":b.type,g=0!==d?c:f;if(h.file&&0!==d){if(null!=a.ArrayBuffer&&g instanceof a.ArrayBuffer){for(var j="",m=new a.Uint8Array(g),n=0;n<g.byteLength;n+=1)j+=String.fromCharCode(m[n]);i.reject(j)}else if(null!=a.Blob&&g instanceof a.Blob){var o=new a.FileReader;o.onload=function(a){i.reject(a.target.result)},o.readAsText(g)}}else{var p=g||f||null;Array.isArray(p)&&(p=new e.Error("Received an array as a response with a status code of "+d+". A JSON object is expected as a response to requests that result in an error status code.")),i.reject(p)}})}},f.debug("Initiating a network request.",b,c,d,g,h),x(d)&&!(null!=a.ArrayBuffer&&d instanceof a.ArrayBuffer||null!=a.Blob&&d instanceof a.Blob)&&(d=JSON.stringify(d)),j.send(d),!aa.supportsTimeout&&j.timeout&&(k=a.setTimeout(function(){j.abort()},j.timeout));var o={cancel:function(){l=!0,j.abort()}};return h.handler&&"function"==typeof h.handler&&h.handler(o),i.promise}};e.Persistence.Net.use(aa);var ba={};if("undefined"!=typeof localStorage){var ca=e.Defer.resolve(null);ba={_destroy:function(a){return ca=ca.then(function(){return localStorage.removeItem(a),e.Defer.resolve(null)})},_get:function(a){return ca=ca.then(function(){var b=localStorage.getItem(a);return e.Defer.resolve(b?JSON.parse(b):null)})},_save:function(a,b){return ca=ca.then(function(){return localStorage.setItem(a,JSON.stringify(b)),e.Defer.resolve(null)})}},F.use(ba)}var da="push";return e.Error.PUSH_ERROR="PushError",p[e.Error.PUSH_ERROR]={name:e.Error.PUSH_ERROR,description:"There is a problem with push notifications.",debug:""},e.Push={register:function(b,c){var d;f.debug("Registering a device to receive push notifications.",arguments),c=c||{};var g=e.getActiveUser();if(null===g&&null==c.userId)return d=new e.Error("options argument must contain: userId."),t(e.Defer.reject(d),c);if(null==a.device)return d=r(e.Error.PUSH_ERROR,{description:"Unable to obtain the device platform.",debug:"Did you install the Cordova Device plugin?"}),t(e.Defer.reject(d),c);var h=e.Persistence.Net.create({namespace:da,id:"register-device",data:{platform:a.device.platform.toLowerCase(),framework:"phonegap",deviceId:b,userId:null!=g?null:c.userId},auth:null!=g?H.Session:H.Master},c);return h.then(function(a){f.debug("Registered the device to receive push notifications.",a)},function(a){f.error("Failed to register the device to receive push notifications.",a)}),t(h,c)},unregister:function(b,c){var d;f.debug("Registering a device to receive push notifications.",arguments),c=c||{};var g=e.getActiveUser();if(null===g&&null==c.userId)return d=new e.Error("options argument must contain: userId."),t(e.Defer.reject(d),c);if(null==a.device)return d=r(e.Error.PUSH_ERROR,{description:"Unable to obtain the device platform.",debug:"Did you install the Cordova Device plugin?"}),t(e.Defer.reject(d),c);var h=e.Persistence.Net.create({namespace:da,id:"unregister-device",data:{platform:a.device.platform.toLowerCase(),framework:"phonegap",deviceId:b,userId:null!=g?null:c.userId},auth:null!=g?H.Session:H.Master},c);return h.then(function(a){f.debug("Unregistered the device from receiving push notifications.",a)},function(a){f.error("Failed to unregister the device from receiving push notifications.",a)}),t(h,c)}},e},c=b();"object"==typeof module&&"object"==typeof module.exports?module.exports=c:"function"==typeof define&&define.amd?define("kinvey",[],function(){return c}):a.Kinvey=c}).call(this);
